# Author: Carsten Sachse 
# Copyright: EMBL (2010 - 2018)
# License: see license.txt for details
from EMAN2 import EMData, Util, Transform, EMUtil
from filter import fit_tanh, filt_tanl, filt_table
from fundamentals import rot_shift2D
from random import randint
from spring.csinfrastr.csreadinput import OptHandler
from spring.segment2d.segment import Segment
from spring.segment2d.segmentalign2d_prep import SegmentAlign2dPar, SegmentAlign2dPreparation
from spring.segment2d.segmentexam import SegmentExam
from statistics import fsc
from tabulate import tabulate
from utilities import compose_transform2
import numpy as np
import os

        
class SegmentAlign2dImagesToReferences(SegmentAlign2dPreparation):
    def put_random_image_in_reference_image_container(self, reference_stack, assigned_images, each_ref_id,
    alignment_stack_name):
        
        stack_image_count = EMUtil.get_image_count(alignment_stack_name)
        assigned_images[each_ref_id] = []
        random_image_id = randint(0, stack_image_count - 1)
        assigned_images[each_ref_id].append(random_image_id)
        
        random_image = EMData()
        random_image.read_image(alignment_stack_name, random_image_id)
        reference_stack[each_ref_id] = reference_stack[each_ref_id]._replace(odd_average=random_image)
                
        return reference_stack
    

    def compute_average_and_normalize(self, reference_stack, each_ref_id):
        updated_average = reference_stack[each_ref_id].odd_average + reference_stack[each_ref_id].even_average
        
        updated_average /= float(reference_stack[each_ref_id].number_of_images[''])
        reference_stack[each_ref_id] = reference_stack[each_ref_id]._replace(total_average=updated_average) 
        
        return reference_stack
                

    def compute_variance_and_normalize(self, reference_stack, each_ref_id):
        cls_avg = reference_stack[each_ref_id].total_average
        cls_var = reference_stack[each_ref_id].variance
        member_count = reference_stack[each_ref_id].number_of_images['']
        
        var = (cls_var - cls_avg * cls_avg * member_count) / (member_count - 1)
        reference_stack[each_ref_id] = reference_stack[each_ref_id]._replace(variance=var)
        
        return reference_stack
    
    
    def calculate_averages(self, reference_stack, iteration_id, each_ref_id):
        
        frsc = fsc(reference_stack[each_ref_id].odd_average, reference_stack[each_ref_id].even_average, 1.0,
        'drm_it{0:03}_ref{0:04}.dat'.format(iteration_id, each_ref_id))
        
        reference_stack = self.compute_average_and_normalize(reference_stack, each_ref_id)
        reference_stack = self.compute_variance_and_normalize(reference_stack, each_ref_id)
            
        return frsc, reference_stack
    

class SegmentAlign2dAlign(SegmentAlign2dImagesToReferences):
    def low_pass_filter_reference_according_to_frc(self, total_average, frc_line):
        """
        * Function to filter low-resolution reference image with a hyperbolic tangent that was fitted \
        against fourier ring correlation
         
        Prepare the reference in 2D alignment, i.e., low-pass filter and center.
        Input: list ref_data
        2 - raw average
        3 - fsc result
        Output: filtered, centered, and masked reference image
        apply filtration (FRC) to reference image:
        
        """
        frequency_cutoff, filter_falloff_width = fit_tanh(frc_line)
        filter_falloff_width = min(filter_falloff_width, 0.12)
        frequency_cutoff = max(min(0.4,frequency_cutoff),0.2)
        msg = 'Tangent filter:  cut-off frequency = %10.3f  fall-off = %10.3f' \
                                        %(frequency_cutoff, filter_falloff_width)
        self.log.ilog(msg)
        
        tanl_filtered_reference = filt_tanl(total_average, frequency_cutoff, filter_falloff_width)
        
        return  tanl_filtered_reference


    def generate_reference_rings_from_image(self, reference_image, polar_interpolation_parameters, ring_weights,
    image_dimension, full_circle_mode='F'):
        center_x = image_dimension / 2 + 1
        center_y = center_x

        cimage = Util.Polar2Dm(reference_image, center_x, center_y, polar_interpolation_parameters, full_circle_mode)
        Util.Normalize_ring(cimage, polar_interpolation_parameters)
        Util.Frngs(cimage, polar_interpolation_parameters)
        Util.Applyws(cimage, polar_interpolation_parameters, ring_weights)
        
        return cimage


    def make_rings_and_prepare_cimage_header(self, image_dimension, polar_interpolation_parameters, ring_weights,
    reference_image):
        cimage = self.generate_reference_rings_from_image(reference_image, polar_interpolation_parameters, ring_weights,
        image_dimension)

        phi = 0
        theta = 90.0
        psi = 270.0
        n1 = np.sin(np.deg2rad(theta)) * np.cos(np.deg2rad(phi))
        n2 = np.sin(np.deg2rad(theta)) * np.sin(np.deg2rad(phi))
        n3 = np.cos(np.deg2rad(theta))
        cimage.set_attr_dict({'n1':n1, 'n2':n2, 'n3':n3})
        cimage.set_attr('phi', phi)
        cimage.set_attr('theta', theta)
        cimage.set_attr('psi', psi)

        return cimage


    def prepare_reference_images_for_alignment(self, mask, reference_stack):
        self.log.fcttolog()
        ringref = []
        references_image_count = len(reference_stack)
        
        image_dimension = mask.get_xsize()
        
        self.log.dlog('Mask pixel dimensions: {0} and reference pixel dimensions: {1}'.format(image_dimension,
        reference_stack[0].total_average.get_xsize()))
                      
        polar_interpolation_parameters, ring_weights = self.prepare_empty_rings(1, image_dimension / 2 - 2, 1)
        
        for each_ref_id in list(range(references_image_count)):
            reference_stack[each_ref_id].total_average.process_inplace('normalize.mask', {'mask':mask,
            'no_sigma':1})
            
            reference_image = reference_stack[each_ref_id].total_average * mask
            
            cimage = self.make_rings_and_prepare_cimage_header(image_dimension, polar_interpolation_parameters,
            ring_weights, reference_image)
            
            ringref.append(cimage) 

            reference_stack[each_ref_id].odd_average.to_zero()
            reference_stack[each_ref_id].even_average.to_zero()
            reference_stack[each_ref_id].total_average.to_zero()
            reference_stack[each_ref_id].variance.to_zero()
            reference_stack[each_ref_id].number_of_images[''] = 0
            
        return ringref, polar_interpolation_parameters, ring_weights, reference_stack
    

    def define_parameters_for_alignment(self, alignment_stack_name, alignment_info, ringref):
        references_image_count = len(ringref)
        assigned_images = [[] for each_reference_img in list(range(references_image_count))]
        align_img = EMData()
        align_img.read_image(alignment_stack_name)
        center_x = center_y = align_img.get_xsize() / 2 + 1
        determined_params = []
        image_nt = self.get_image_list_named_tuple()
        dummy_transform = Transform({'type':'spider', 'phi':0.0, 'theta':90.0, 'psi':270.0})
        
        x_range = alignment_info.x_range
        y_range = alignment_info.y_range
        x_limit = self.x_limit_A / alignment_info.pixelsize
        y_limit = self.y_limit_A / alignment_info.pixelsize

        translation_step = 1
        
        return align_img, (x_range, y_range), (x_limit, y_limit), translation_step, center_x, center_y, dummy_transform,\
        image_nt, determined_params, assigned_images


    def limit_search_range_based_on_previous_alignment(self, local_prev_shift_x, x_limit, fine_x_range):
        """
        >>> from spring.segment2d.segmentalign2d import SegmentAlign2d
        >>> s = SegmentAlign2d()
        >>> s.limit_search_range_based_on_previous_alignment(2.75, 3, 0.5)
        2.5
        >>> s.limit_search_range_based_on_previous_alignment(2.25, 3, 0.5)
        2.25
        >>> s.limit_search_range_based_on_previous_alignment(-2.75, 3, 0.5)
        -2.5
        >>> s.limit_search_range_based_on_previous_alignment(2.05, 3, 3)
        0
        >>> s.limit_search_range_based_on_previous_alignment(2.05, 5, 3)
        2
        """
        x_range_border = x_limit - fine_x_range
        if local_prev_shift_x > x_range_border:
            local_prev_shift_x = x_range_border
        elif local_prev_shift_x < -x_range_border:
            local_prev_shift_x = -x_range_border

        return local_prev_shift_x


    def perform_coarse_restrained_alignment(self, alignment_stack_name, ringref, polar_interpolation_parameters,
    alignment_info, refine_locally, full_circle_mode, align_img, search_ranges, search_limits, translation_step,
    center_x, center_y, each_image):
    
        x_range, y_range = search_ranges
        x_limit, y_limit = search_limits
        align_img.read_image(alignment_stack_name, each_image.local_id)
        shift_x, shift_y = each_image.shift_x / alignment_info.pixelsize, each_image.shift_y / alignment_info.pixelsize
        
        shift_x = self.limit_search_range_based_on_previous_alignment(shift_x, x_limit, x_range)
        shift_y = self.limit_search_range_based_on_previous_alignment(shift_y, y_limit, y_range)
            
        if refine_locally:
            sxb = 0.0
            syb = 0.0
        else:
            [angt, sxst, syst, mirror, xiref, peakt] = Util.multiref_polar_ali_2d_delta(align_img, ringref, x_range,
            y_range, translation_step, full_circle_mode, polar_interpolation_parameters, center_x + shift_x, 
            center_y + shift_y, each_image.inplane_angle, self.delta_psi)
            
            angb, sxb, syb, ct = compose_transform2(0.0, sxst, syst, 1, -angt, 0.0, 0.0, 1)
            
        local_prev_shift_x = shift_x - sxb
        local_prev_shift_y = shift_y - syb
        
        return local_prev_shift_x, local_prev_shift_y


    def perform_fine_alignment(self, ringref, polar_interpolation_parameters, alignment_info, full_circle_mode,
    align_img, search_ranges, search_limits, translation_step, center_x, center_y, dummy_transform, image_nt,
    determined_params, each_image, local_prev_shift_x, local_prev_shift_y):
        x_range, y_range = search_ranges
        x_limit, y_limit = search_limits
    
        zoom_factor = 5.0
        step_angle = 0.05
        fine_step_angle = np.sin(np.deg2rad(step_angle / zoom_factor))
        fine_x_range = x_range / zoom_factor
        fine_y_range = y_range / zoom_factor
        align_img.set_attr('xform.projection', dummy_transform)
        
        local_prev_shift_x = self.limit_search_range_based_on_previous_alignment(local_prev_shift_x, x_limit,
        fine_x_range)
            
        local_prev_shift_y = self.limit_search_range_based_on_previous_alignment(local_prev_shift_y, y_limit,
        fine_y_range)
            
        [angt, sxst, syst, mirror, xref, peakt] = Util.multiref_polar_ali_2d_local(align_img, ringref, fine_x_range,
        fine_y_range, translation_step / zoom_factor, fine_step_angle, full_circle_mode, polar_interpolation_parameters,
        center_x + local_prev_shift_x, center_y + local_prev_shift_y)
        
        matched_reference = int(xref)
        
        angb, sxb, syb, ct = compose_transform2(0.0, sxst, syst, 1, -angt, 0.0, 0.0, 1)
        refined_shift_x = local_prev_shift_x - sxb
        refined_shift_y = local_prev_shift_y - syb
        
        new_params = image_nt(each_image.stack_id, each_image.local_id, matched_reference, refined_shift_x *
        alignment_info.pixelsize, refined_shift_y * alignment_info.pixelsize, angt, peakt, mirror)
        
        determined_params.append(new_params)
        
        return angt, refined_shift_x, refined_shift_y, mirror, matched_reference, determined_params


    def determine_odd_and_even_average_including_variance(self, align_img, reference_stack, assigned_images, each_image,
    angt, refined_shift_x, refined_shift_y, mirror, matched_reference):
        alphan, sxn, syn = Segment().convert_shift_rotate_to_rotate_shift_order(angt, -refined_shift_x, -refined_shift_y)
        
        img_align_params_applied = rot_shift2D(align_img, alphan, sxn, syn, mirror)
        
        odd = each_image.stack_id % 2
        if odd:
            Util.add_img(reference_stack[matched_reference].odd_average, img_align_params_applied)
        elif not odd:
            Util.add_img(reference_stack[matched_reference].even_average, img_align_params_applied)
            
        Util.add_img2(reference_stack[matched_reference].variance, img_align_params_applied)
        assigned_images[matched_reference].append(each_image.stack_id)
        reference_stack[matched_reference].number_of_images[''] += 1

        return reference_stack
    

    def log_alignment_params(self, previous_params, determined_params):
        """
        >>> from spring.segment2d.segmentalign2d import SegmentAlign2d
        >>> s = SegmentAlign2d()
        >>> param_nt = s.get_image_list_named_tuple()
        >>> a = b = [param_nt(1, 1, 3, 0, 0, 0, 0, 1)]
        >>> SegmentAlign2d().log_alignment_params(a, b)
        u'  stack_id    local_id    ref_id    shift_x    shift_y    inplane_angle    peak    mirror  cycle\\n----------  ----------  --------  ---------  ---------  ---------------  ------  --------  ----------\\n         1           1         3          0          0                0       0         1  previous\\n         1           1         3          0          0                0       0         1  determined'
        """
        log_info = []
        for each_prev_param, each_det_param in zip(previous_params, determined_params):
            log_info += [list(each_prev_param) + ['previous']]
            log_info += [list(each_det_param) + ['determined']]
            
        msg = tabulate(log_info, list(each_prev_param._fields) + ['cycle'])
        self.log.tlog('The following alignment parameters were determined:\n{0}'.format(msg))
        
        return msg
            
        
    def align_images_to_references(self, alignment_stack_name, reference_stack, previous_params, ringref,
    polar_interpolation_parameters, ring_weights, alignment_info, refine_locally=True, full_circle_mode='F'):
        self.log.fcttolog()
        self.log.in_progress_log()
        
        align_img, search_ranges, search_limits, translation_step, center_x, center_y, dummy_transform, image_nt, \
        determined_params, assigned_images = self.define_parameters_for_alignment(alignment_stack_name, alignment_info,
        ringref)
        
        for each_image in previous_params:
            local_prev_shift_x, local_prev_shift_y = self.perform_coarse_restrained_alignment(alignment_stack_name,
            ringref, polar_interpolation_parameters, alignment_info, refine_locally, full_circle_mode, align_img,
            search_ranges, search_limits, translation_step, center_x, center_y, each_image)
            
            angt, refined_shift_x, refined_shift_y, mirror, matched_reference, determined_params = \
            self.perform_fine_alignment(ringref, polar_interpolation_parameters, alignment_info, full_circle_mode,
            align_img, search_ranges, search_limits, translation_step, center_x, center_y, dummy_transform, image_nt,
            determined_params, each_image, local_prev_shift_x, local_prev_shift_y)
            
            reference_stack = self.determine_odd_and_even_average_including_variance(align_img, reference_stack,
            assigned_images, each_image, angt, refined_shift_x, refined_shift_y, mirror, matched_reference)
        
        self.log_alignment_params(previous_params, determined_params)
        
        return assigned_images, determined_params, reference_stack
    

class SegmentAlign2dPostAlign(SegmentAlign2dAlign):
    def filter_references_if_requested(self, reference_stack, frc_line, each_ref_id):
        if self.low_pass_filter_option or self.high_pass_filter_option or self.custom_filter_option is True \
        or self.bfactor != 0:
            filter_coefficients = self.prepare_filter_function(self.high_pass_filter_option,
            self.high_pass_filter_cutoff, self.low_pass_filter_option, self.low_pass_filter_cutoff,
            self.pixelsize, self.image_dimension, 0.08, self.custom_filter_option, self.custom_filter_file,
            self.bfactor)
            
            filtered_average = filt_table(reference_stack[each_ref_id].total_average, filter_coefficients)
            reference_stack[each_ref_id] = reference_stack[each_ref_id]._replace(total_average=filtered_average)
            
        if self.frc_filter_option is True and frc_line is not None:
            updated_average = \
            self.low_pass_filter_reference_according_to_frc(reference_stack[each_ref_id].total_average, frc_line)
            
            reference_stack[each_ref_id] = reference_stack[each_ref_id]._replace(total_average=updated_average)
        
        return reference_stack
            

    def pass_alignment_parameters_from_reference_groups_to_images(self, reference_stack, alignment_info,
    assigned_images, mask, alignment_stack_name, reference_aligned):
        self.log.fcttolog()
        similarity_criterion = 0.0
    
        references_image_count = len(reference_stack)
        log_info = []
        for each_ref_id in list(range(references_image_count)):
            log_info += [[each_ref_id, reference_stack[each_ref_id].number_of_images['']]]
            
            if reference_stack[each_ref_id].number_of_images[''] <= 3: 
                reference_stack = self.put_random_image_in_reference_image_container(reference_stack, assigned_images,
                each_ref_id, alignment_stack_name)
                frc_line = None
            else:
                if self.update_references:
                    frc_line, reference_stack = self.calculate_averages(reference_stack, alignment_info.iteration_id,
                    each_ref_id)
        
            reference_stack[each_ref_id].total_average.set_attr('ave_n',
            reference_stack[each_ref_id].number_of_images[''])
            
            members = assigned_images[each_ref_id]
            reference_stack[each_ref_id].total_average.set_attr('members', members)
            
            if self.update_references:
                self.write_aligned_unfiltered_averages_and_variances(reference_stack, reference_aligned, each_ref_id)
            
                reference_stack = self.filter_references_if_requested(reference_stack, frc_line, each_ref_id)
                    
                new_reference_image = 'aqm{0:03}.hdf'.format(alignment_info.iteration_id)
                reference_stack[each_ref_id].total_average.write_image(new_reference_image, each_ref_id)
            
            similarity_criterion += reference_stack[each_ref_id].total_average.cmp(
                        'dot', reference_stack[each_ref_id].total_average, {'negative':0, 'mask':mask})
            
        msg = tabulate(log_info, ['reference_group', 'particle_count'])
        self.log.tlog('The following references have matched the number of the images:\n{0}'.format(msg))
        
        return similarity_criterion, reference_stack


    def generate_temp_bin_name(self, tempdir, stack_name, spec='ali'):
        temp_alignment_stack = os.path.join(tempdir,
        '{0}binned{1}{2}'.format(os.path.splitext(os.path.basename(stack_name))[0], spec, os.path.splitext(stack_name)[-1]))
        
        return temp_alignment_stack
    

    def write_out_aligned_averages_and_adapt_scales_from_previous_cycle(self, reference_stack_name, reference_stack,
    segsizepix, previous_binfactor, current_binfactor):
        
        scale_factor = previous_binfactor / float(current_binfactor)
        for each_ref_id, each_ref in enumerate(reference_stack):
            img = each_ref.total_average.copy()
            if scale_factor > 1:
                img = Util.pad(img, segsizepix, segsizepix, 1, 0, 0, 0)
            elif scale_factor < 1:
                img = Util.window(img, segsizepix, segsizepix, 1, 0, 0, 0)
            
            if scale_factor != 1:
                img.scale(scale_factor)
            img.write_image(reference_stack_name, each_ref_id)
        
        if scale_factor > 1:
            self.log.dlog('References were padded to {0} pixel dimensions'.format(segsizepix))
        elif scale_factor > 1:
            self.log.dlog('References were windowed to {0} pixel dimensions'.format(segsizepix))
        else:
            self.log.dlog('References are of {0} pixel dimensions'.format(segsizepix))
            
        return reference_stack_name
        
            
class SegmentAlign2d(SegmentAlign2dPostAlign):
    
    def define_previous_params_and_refine_locally(self, images_info, determined_params, previous_binfactor, align_id,
    each_info):
        if align_id == 0:
            previous_params = images_info
        else:
            previous_params = determined_params
            
        if previous_binfactor != each_info.binfactor:
            refine_locally = False
        else:
            refine_locally = True
            
        return previous_params, refine_locally
    

    def bin_references_and_images(self, alignment_stack_name, reference_stack_name, reference_stack, alignment_info,
    image_ids, align_id, previous_binfactor):
        segsizepix = self.image_dimension
        
        temp_alignment_stack = self.generate_temp_bin_name(self.tempdir, alignment_stack_name)
        
        if previous_binfactor != alignment_info.binfactor:
            temp_alignment_stack, segsizepix, helixwidthpix, pixelsize = \
            SegmentExam().apply_binfactor(alignment_info.binfactor, alignment_stack_name, segsizepix, self.helixwidthpix,
            self.pixelsize, image_ids, temp_alignment_stack)
        
        temp_reference_stack = self.generate_temp_bin_name(self.tempdir, reference_stack_name, 'ref')
        if align_id == 0 or not self.update_references:
            ref_img_list = list(range(EMUtil.get_image_count(reference_stack_name)))
            temp_reference_stack, segsizepix, helixwidthpix, pixelsize = \
            SegmentExam().apply_binfactor(alignment_info.binfactor, reference_stack_name, segsizepix, self.helixwidthpix,
            self.pixelsize, ref_img_list, temp_reference_stack)
        else:
            image = EMData()
            image.read_image(temp_alignment_stack)
            segsizepix = image.get_xsize()
            
            temp_reference_stack = \
            self.write_out_aligned_averages_and_adapt_scales_from_previous_cycle(temp_reference_stack, reference_stack,
            segsizepix, previous_binfactor, alignment_info.binfactor)
        
        reference_stack = self.prepare_reference_stack(temp_reference_stack)
        
        helixwidthpix = int(self.helixwidthpix / float(alignment_info.binfactor))
        helixheightpix = int(self.helixheightpix / float(alignment_info.binfactor))
        bin_mask = self.prepare_mask(helixwidthpix, helixheightpix, segsizepix)
        
        self.log.dlog('Internal mask prepared: segmentsize {0}: width x height {1} x {2} pixels. (Align_id {3})'.\
        format(segsizepix, helixwidthpix, helixheightpix, align_id))
        
        return temp_alignment_stack, reference_stack, bin_mask
    

    def write_aligned_unfiltered_averages_and_variances(self, reference_stack, reference_aligned, each_ref_id):
        new_variance_image_stack = '{0}_var{1}'.format(os.path.splitext(reference_aligned)[0],
        os.path.splitext(reference_aligned)[-1])
        
        reference_stack[each_ref_id].total_average.write_image(reference_aligned, each_ref_id)
        reference_stack[each_ref_id].variance.write_image(new_variance_image_stack, each_ref_id)


    def generate_aligned_output_file_and_update_header_of_input(self, determined_params):
        img = EMData()
        for each_image in determined_params:
            img.read_image(self.infile, each_image.stack_id)
            refined_shift_x = each_image.shift_x / self.pixelsize
            refined_shift_y = each_image.shift_y / self.pixelsize
            
            alphan, sxn, syn = Segment().convert_shift_rotate_to_rotate_shift_order(each_image.inplane_angle, 
            -refined_shift_x, -refined_shift_y)
            
            align_img = rot_shift2D(img, alphan, sxn, syn, each_image.mirror)
            align_img.del_attr('xform.align2d')
            align_img.set_attr('active', 1)
            align_img.write_image(self.outfile, each_image.stack_id)

            trans_2d = Transform({'type':'2d', 'alpha':alphan, 'tx':sxn, 'ty':syn, 'mirror':int(each_image.mirror)})
            img.set_attr('xform.align2d', trans_2d)
            img.set_attr('assign', each_image.ref_id)
            img.write_image(self.infile, each_image.stack_id)
            

    def cleanup_segmentalign2d(self):
        os.remove(self.generate_temp_bin_name(self.tempdir, self.alignment_stack_name))
        os.remove(self.generate_temp_bin_name(self.tempdir, self.reference_stack_name, 'ref'))
        os.rmdir(self.tempdir)


    def perform_iterative_alignment(self, alignment_info, images_info):
        image_ids = [each_image.stack_id for each_image in images_info]
        
        determined_params = None
        previous_binfactor = 0
        reference_stack = None
        
        for align_id, each_info in enumerate(alignment_info):
            
            previous_params, refine_locally = self.define_previous_params_and_refine_locally(images_info,
            determined_params, previous_binfactor, align_id, each_info)
                
            alignment_stack_name, reference_stack, bin_mask = \
            self.bin_references_and_images(self.alignment_stack_name, self.reference_stack_name, reference_stack,
            each_info, image_ids, align_id, previous_binfactor)
            
            ringref, polar_interpolation_parameters, ring_weights, reference_stack = \
            self.prepare_reference_images_for_alignment(bin_mask, reference_stack)
            
            self.log.plog(80 * (align_id + 0.5) / len(alignment_info) + 10)
            
            assigned_images, determined_params, reference_stack = \
            self.align_images_to_references(alignment_stack_name, reference_stack, previous_params, ringref,
            polar_interpolation_parameters, ring_weights, each_info, refine_locally)
            
            similarity_criterion, reference_stack = \
            self.pass_alignment_parameters_from_reference_groups_to_images(reference_stack, each_info, assigned_images,
            bin_mask, alignment_stack_name, self.aligned_averages)
            
            msg = 'ITERATION #{0}        criterion = {1}'.format(each_info.iteration_id, similarity_criterion)
            self.log.tlog(msg)
 
            self.log.plog(80 * (align_id + 1) / len(alignment_info) + 10)
            previous_binfactor = each_info.binfactor
            
        self.generate_aligned_output_file_and_update_header_of_input(determined_params)
            
        self.log.ilog('Alignment parameters were applied to new output file {0}.'.format(self.outfile))
        
        self.cleanup_segmentalign2d()
        if not self.reference_option:
            os.remove(self.reference_stack_name)
        self.log.endlog(self.feature_set)
        
        
    def perform_segmentalign2d(self):
        alignment_info, images_info = self.prepare_alignment()
        self.perform_iterative_alignment(alignment_info, images_info)
        
        
def main():
    # Option handling
    parset = SegmentAlign2dPar()
    mergeparset = OptHandler(parset)

    ######## Program
    stack = SegmentAlign2d(mergeparset)
    stack.perform_segmentalign2d()


if __name__ == '__main__':
    main()



from setuptools import setup, find_packages

def read_file(txt_file):
    with open(txt_file) as f:
        return f.read()

setup(name='emspring',
      version='0.86.1661',
      description='Electron Microscopy Single-Particle Based Helical Reconstruction',
      long_description=read_file('README.rst') + 2 * '\n' +  read_file('CHANGES.rst') + 2 * '\n' + read_file('license.txt'),
      keywords='Electron cryomicroscopy, cryo-EM, image processing, SPARX, EMAN2, Helical assembly, ' + \
      '3D reconstruction, helical symmetry',
      author='Carsten Sachse',
      author_email='carsten.sachse@gmail.com',
      url='http://www.sachse.embl.de/emspring',
      license='Modified BSD License',
    classifiers = [
    'Development Status :: 5 - Production/Stable',
    'Operating System :: Unix',
    'Environment :: X11 Applications :: Qt',
    'Intended Audience :: Science/Research',
    'License :: OSI Approved :: BSD License',
    'Programming Language :: Python',
    'Topic :: Scientific/Engineering :: Image Recognition',
    ],

      packages=find_packages(exclude=['ez_setup', 'examples', 'tests']),
      include_package_data=True,
      zip_safe=False,
      install_requires=[
          # -*- Extra requirements: -*-
      ],
      entry_points = {
        'console_scripts': [
       # -*- Entry points: -*-
            'springenv = spring.csinfrastr.cslaunch:main',
            'spring = spring.springgui.spring_launch:main',
            'scansplit = spring.micprgs.scansplit:main',
            'scansplit_mpi = spring.micprgs.scansplit_mpi:main',
            # scanspring
            'micexam = spring.micprgs.micexam:main',
            'micexam_mpi = spring.micprgs.micexam_mpi:main',
            'micctfdetermine = spring.micprgs.micctfdetermine:main',
            'micctfdetermine_mpi = spring.micprgs.micctfdetermine_mpi:main',
            'michelixtrace = spring.micprgs.michelixtrace:main',
            'michelixtrace_mpi = spring.micprgs.michelixtrace_mpi:main',
            'michelixtracegrid = spring.micprgs.michelixtracegrid:main',
            'scanlinefit = spring.micprgs.scanlinefit:main',
            'scandotfit = spring.micprgs.scandotfit:main',
            'scanrowcolcorr = spring.micprgs.scanrowcolcorr:main',
            # spring2d
            'segment = spring.segment2d.segment:main',
            'segment_mpi = spring.segment2d.segment_mpi:main',
            'segmentexam = spring.segment2d.segmentexam:main',
            'segmentexam_mpi = spring.segment2d.segmentexam_mpi:main',
            'segmentclass = spring.segment2d.segmentclass:main',
            'segmentalign2d = spring.segment2d.segmentalign2d:main',
            'segmentalign2d_mpi = spring.segment2d.segmentalign2d_mpi:main',
            'segclassexam = spring.segment2d.segclassexam:main',
            'segclassexam_mpi = spring.segment2d.segclassexam_mpi:main',
            'segclasslayer = spring.segment2d.segclasslayer:main',
            'seglayer2lattice = spring.segment2d.seglayer2lattice:main',
            'segmentplot = spring.segment2d.segmentplot:main',
            # spring3d
            'segclassreconstruct = spring.segment3d.segclassreconstruct:main',
            'segclassreconstruct_mpi = spring.segment3d.segclassreconstruct_mpi:main',
            'segmentrefine3d = spring.segment3d.refine.sr3d_main:main',
            'segmentrefine3d_mpi = spring.segment3d.refine.sr3d_mpi:main',
            'segrefine3dgrid = spring.segment3d.segrefine3dgrid:main',
            'seggridexplore = spring.segment3d.seggridexplore:main',
            'segrefine3dplot = spring.segment3d.segrefine3dplot:main',
            'segrefine3dcyclexplore = spring.segment3d.segrefine3dcyclexplore:main',
            'segrefine3dinspect = spring.segment3d.segrefine3dinspect:main',
            'segclassmodel = spring.segment3d.segclassmodel:main',
            'segclassmodel_mpi = spring.segment3d.segclassmodel_mpi:main',
            'segmultirefine3d= spring.segment3d.segmultirefine3d:main',
            'segmultirefine3d_mpi = spring.segment3d.segmultirefine3d_mpi:main',
            # particle
            'particlestack = spring.particle2d.particlestack:main',
            'particleclass = spring.particle2d.particleclass:main',
            'particlealign2d = spring.particle2d.particlealign2d:main',
            'particlealign2d_mpi = spring.particle2d.particlealign2d_mpi:main'
        ]

    }
      )



#!/usr/bin/env python
"""
Test module to check particlestack
"""
from multiprocessing import cpu_count
from spring.csinfrastr.csproductivity import TestingDirectory
from spring.particle2d.particlestack import ParticleStack, ParticleStackPar
from spring.tests.csinfrastr.test_csreadinput import EndToEndTest
from spring.tests.segment2d.test_segment import MicrographHelixPreparation, TestSegment
import os


class TestParticleStackPreparation(object):
    def setup_helix_or_particle_dimensions(self):
        self.feature_set.parameters['Particle coordinates'] = self.helix_coordinate_file
        self.feature_set.parameters['Estimated inner and outer particle diameter in Angstrom'] = ((200, 700))

class TestParticleStack(TestParticleStackPreparation, ParticleStack, MicrographHelixPreparation):
    def setup(self):
        self.testingdir = TestingDirectory(os.path.splitext(os.path.basename(__file__))[0])
        self.testingdir.create()

        self.prepare_helix_on_micrograph()
        
        self.feature_set = ParticleStackPar()
        self.setup_of_common_non_mpi_parameters()
        
        self.feature_set.parameters['MPI option']=False
        self.feature_set.parameters['Number of CPUs']=min(8, cpu_count())

        super(TestParticleStack, self).__init__(self.feature_set)

    def do_test_case_ps1(self):
        self.window_particles()

    def teardown(self):
        TestSegment().remove_coordfile_micrographdir_and_linkfiles(self.helix_coordinate_file, self.micrograph_test_file)
        os.remove(self.micrograph_test_file)
        os.remove('test_output_stack-6xbin.hdf')
        self.testingdir.remove()


class TestParticleStackMore(TestParticleStack):
    def do_end_to_end_test_pc_inputfile(self):
        EndToEndTest().do_end_to_end_inputfile(self.feature_set)

    def do_end_to_end_test_pc_prompt(self):
        EndToEndTest().do_end_to_end_prompt(self.feature_set)

class TestParticleStackMpi(TestParticleStack):
    def do_end_to_end_test_pc_inputfile_MPI(self):

        self.feature_set.parameters['MPI option']=True
        self.feature_set.parameters['Number of CPUs']=min(8, cpu_count())
        
        EndToEndTest().do_end_to_end_inputfile(self.feature_set)


def main():
    tsce = TestParticleStack()
    tsce.setup()
    tsce.do_test_case_ps1()
        
if __name__ == '__main__':
    main()



# Author: Carsten Sachse 19-Aug-2017
# Copyright: EMBL (2010 - 2018)
# License: see license.txt for details
"""
Program to optimize michelixtrace by varying michelixtrace parameters systematically on a grid
"""

from collections import OrderedDict
from spring.csinfrastr.csdatabase import SpringDataBase, grid_base, GridTable, GridRefineTable
from spring.csinfrastr.csfeatures import Features
from spring.csinfrastr.cslogger import Logger
from spring.csinfrastr.csproductivity import OpenMpi
from spring.csinfrastr.csreadinput import OptHandler
from spring.micprgs.michelixtrace import MicHelixTracePar, MicHelixTrace
from spring.segment3d.segclassreconstruct import SegClassReconstruct
from spring.segment3d.segrefine3dgrid import SegRefine3dGrid

from sqlalchemy.sql.expression import desc
from tabulate import tabulate

import numpy as np
import os


class MicHelixTraceGridPar(MicHelixTracePar):
    """
    Class to initiate default dictionary with input parameters including help and range values and status dictionary
    """
    def __init__(self):
        # package/program identity
        self.package = 'emspring'       
        self.progname = 'michelixtracegrid'
        self.proginfo = __doc__
        self.code_files = [self.progname]

        self.mictrace_features = Features()
        self.feature_set = self.mictrace_features.setup(self)
        
        self.define_parameters_and_their_properties()
        self.define_program_states()


    def define_parameters_and_their_properties(self):
        super(MicHelixTraceGridPar, self).define_input_and_output_michelixtrace()
        parameters_to_be_varied = self.get_parameters_to_be_varied()

        self.feature_set = self.mictrace_features.set_first_parameter_choice(self.feature_set, parameters_to_be_varied,
        'alpha_threshold')
        
        self.feature_set = self.mictrace_features.set_second_parameter_choice(self.feature_set, parameters_to_be_varied,
        'min_helix_length' )
        
        self.feature_set = self.mictrace_features.set_lower_and_upper_limit_first_parameter(self.feature_set)
        self.feature_set = self.mictrace_features.set_lower_and_upper_limit_second_parameter(self.feature_set)
        self.feature_set = self.mictrace_features.set_first_and_second_parameter_increment(self.feature_set)
        self.feature_set = self.mictrace_features.set_subgrid_option(self.feature_set)
        self.feature_set = self.mictrace_features.set_subgrid_details(self.feature_set)
        self.feature_set = self.mictrace_features.set_continue_grid_option(self.feature_set)
        self.feature_set = self.mictrace_features.set_grid_database_cont(self.feature_set)
        
        super(MicHelixTraceGridPar, self).define_michelixtrace_parameters()
        
    def define_program_states(self):
        super(MicHelixTraceGridPar, self).define_program_states()

    def get_parameters_to_be_varied(self):
        parameter_to_be_varied = [
        'tile_size_power', 'tile_overlap', 'binning_factor', 'alpha_threshold',
        'min_helix_length', 'max_helix_length', 'order_fit'
                                  ]
        return parameter_to_be_varied


class MicHelixTraceGridSetup(MicHelixTrace):
    def __init__(self, parset = None):
        self.log = Logger()
        if parset is not None:
            self.feature_set = parset
            p = self.feature_set.parameters
            self.define_all_michelixtrace_parameters(p)
            
            self.primary_variable = p['First parameter']
            self.secondary_variable = p['Second parameter']
            self.primary_range = p['Lower and upper limit first parameter']
            self.primary_inc, self.second_inc = p['First and second parameter increment']
            self.secondary_range = p['Lower and upper limit second parameter']
            self.subgrid_option = p['Subgrid option']
            self.subgrid = p['Part and number of subgrids']
            self.continue_grid = p['Grid continue option']
            self.grid_database = p['Grid database']
            
        
    def generate_michelixtrace_parameters_while_removing_grid_parameters(self, parameters):
        
        michelixtrace_params = OrderedDict()
        for each_param in parameters:
            if each_param not in ['First parameter', 'Second parameter', 'Lower and upper limit first parameter',
                                  'Lower and upper limit second parameter', 'First and second parameter increment',
                                  'Subgrid option', 'Part and number of subgrids', 'Grid continue option', 
                                  'Grid database']:
                michelixtrace_params[each_param] = parameters[each_param]
        
        return michelixtrace_params
    

    def setup_grid_parameters(self, each_grid_point, parameters, index, one_of_both_parameters):
        if one_of_both_parameters == 'tile_size_power':
            parameters['Tile size power spectrum in Angstrom'] = each_grid_point[index]
        elif one_of_both_parameters == 'tile_overlap':
            parameters['Tile overlap in percent'] = each_grid_point[index]
        elif one_of_both_parameters == 'binning_factor':
            parameters['Binning factor'] = each_grid_point[index]
        elif one_of_both_parameters == 'alpha_threshold':
            parameters['Alpha threshold cc-map'] = each_grid_point[index]
        elif one_of_both_parameters == 'order_fit':
            parameters['Order fit'] = each_grid_point[index]
            
        elif one_of_both_parameters == 'min_helix_length':
            parameters['Minimum and maximum helix length'] = \
            (each_grid_point[index], parameters['Minimum and maximum helix length'][1])
        elif one_of_both_parameters == 'max_helix_length':
            parameters['Minimum and maximum helix length'] = \
            (parameters['Minimum and maximum helix length'][0], each_grid_point[index])
        return parameters
    

    def add_grid_point_to_parameter_dictionary(self, first_param, second_param, each_grid_point, parameters):
        """
        >>> from spring.micprgs.michelixtracegrid import MicHelixTraceGrid
        >>> min_hl = 'min_helix_length'
        >>> params = MicHelixTraceGridPar().parameters
        >>> s = MicHelixTraceGrid()
        >>> s.add_grid_point_to_parameter_dictionary('alpha_threshold', min_hl, [-111, 1.111], params) #doctest: +NORMALIZE_WHITESPACE
        OrderedDict([('Micrographs', 'cs_scan034.tif'), ('Diagnostic plot pattern', 'michelixtracegrid_diag.pdf'), 
        ('First parameter', 'alpha_threshold'), ('Second parameter', 'min_helix_length'), 
        ('Lower and upper limit first parameter', (1.4, 1.9)), ('Lower and upper limit second parameter', (22.0, 24.0)), 
        ('First and second parameter increment', (0.1, 0.3)), ('Subgrid option', False), 
        ('Part and number of subgrids', (1, 3)), ('Grid continue option', False), ('Grid database', 'grid.db'), 
        ('Helix reference', 'helix_reference.hdf'), ('Invert option', False), ('Estimated helix width in Angstrom', 200), 
        ('Pixel size in Angstrom', 1.163), ('Binning option', True), ('Binning factor', 4), 
        ('Tile size power spectrum in Angstrom', 500), ('Tile overlap in percent', 80), ('Alpha threshold cc-map', -111), 
        ('Absolute threshold option cc-map', False), ('Absolute threshold cc-map', 0.2), ('Order fit', 2), 
        ('Minimum and maximum helix length', (1.111, 1500)), ('Pruning cutoff bending', 2.0), 
        ('Box file coordinate step', 70.0), ('Compute performance score', False), ('Parameter search option', False), 
        ('Manually traced helix file', 'mic.box'), ('MPI option', True), ('Number of CPUs', 2), 
        ('Temporary directory', '/tmp')])
        """
        for index, one_of_both_parameters in enumerate([first_param, second_param]):
            if one_of_both_parameters is not None:
                parameters = self.setup_grid_parameters(each_grid_point, parameters, index,
                one_of_both_parameters)
        
        return parameters


    def setup_parameters_for_each_grid_point(self, first_param, second_param, each_grid_point, parameters):
        parameters = self.add_grid_point_to_parameter_dictionary(first_param, second_param, each_grid_point, parameters)
        run_name = SegRefine3dGrid().generate_file_name_for_grid_search(first_param, second_param, each_grid_point)
                
        parameter_file = Features().write_parfile(parameters, run_name, tag=False)
        
        return parameter_file, run_name
    
    
class MicHelixTraceGrid(MicHelixTraceGridSetup):
    
    def get_variables_ranges_and_increments(self, grid_name):
        if self.continue_grid:
            last_grid_cycle_id, ref_grid = \
            SegRefine3dGrid().get_grid_parameters_and_final_grid_entry(self.grid_database, grid_name)
            
            primary_variable = ref_grid.primary_variable
            secondary_variable = ref_grid.secondary_variable
            primary_range = ref_grid.primary_min, ref_grid.primary_max
            primary_inc = ref_grid.primary_inc
            secondary_range = ref_grid.second_min, ref_grid.second_max
            second_inc = ref_grid.second_inc
        else:
            last_grid_cycle_id = 0
            primary_variable = self.primary_variable
            primary_range = self.primary_range
            primary_inc = self.primary_inc
            secondary_variable = self.secondary_variable
            secondary_range = self.secondary_range
            second_inc = self.second_inc
 
            if primary_variable == secondary_variable or secondary_variable == 'none':
                secondary_variable = None
                secondary_range = (None, None)
                second_inc = None
                secondary_range = self.iteration_count, self.iteration_count
                second_inc = 0
 
        return primary_range, primary_inc, secondary_range, second_inc, last_grid_cycle_id, primary_variable, \
        secondary_variable
 
 
    def setup_grid_sequence(self):
        grid_name = SegRefine3dGrid().get_grid_database_name(self.subgrid_option)
             
        primary_range, primary_inc, secondary_range, second_inc, last_grid_cycle_id, primary_variable, \
        secondary_variable = self.get_variables_ranges_and_increments(grid_name)
             
        if not 'alpha_threshold' in [primary_variable, secondary_variable]:
            grid_sequence = \
            SegClassReconstruct().generate_rise_rotation_or_pitch_unitnumber_pairs_for_symmetry_grid(primary_range,
            primary_inc, secondary_range, second_inc)
            
        else:
            primary_series, second_series = SegClassReconstruct().generate_unique_rise_rotation_or_pitch_unitnumber_arrays(primary_range,
            primary_inc, secondary_range, second_inc)
        
            if primary_variable == 'alpha_threshold':
                primary_series = \
                np.logspace(np.log10(min(primary_series)), np.log10(max(primary_series)), len(primary_series)).tolist()
            elif secondary_variable == 'alpha_threshold':
                second_series = \
                np.logspace(np.log10(min(second_series)), np.log10(max(second_series)), len(second_series)).tolist()

            grid_sequence =\
            SegClassReconstruct().convert_rise_rotation_or_pitch_unitnumber_series_to_grid_of_tuples(primary_series,
            second_series)
             
        grid_sequence = grid_sequence.ravel()
        grid_sequence = SegRefine3dGrid().split_grid_if_demanded(self.subgrid_option, self.subgrid, grid_sequence)
         
        grid_str = [(each_id, each_first, each_second) for each_id, (each_first, each_second) in enumerate(grid_sequence)]
        msg = tabulate(grid_str, ['local_grid_id', primary_variable, secondary_variable])
        self.log.ilog('The following grid combinations will be run:\n{0}'.format(msg))
             
        return grid_sequence, last_grid_cycle_id, primary_variable, secondary_variable, grid_name


    def generate_cmdline_for_each_grid_point(self, parameter_file):
        """
        >>> from spring.micprgs.michelixtracegrid import MicHelixTraceGrid
        >>> s = MicHelixTraceGrid()
        >>> s.generate_cmdline_for_each_grid_point('alpha_threshold_1e-10_min_helix_length_500.par')
        'michelixtrace --d alpha_threshold_1e-10_min_helix_length_500 --f alpha_threshold_1e-10_min_helix_length_500.par'
        """
        cmdline = 'michelixtrace --d {0} --f {1}'.format(os.path.splitext(parameter_file)[0],
        parameter_file)
        
        return cmdline
    

    def make_database_entries(self, run_name, each_grid_point, grid_session, comparison_measures):
        grid_cycle = GridRefineTable()
        rundir_name = os.path.abspath(run_name)
        grid_cycle.dirname = rundir_name
        grid_cycle.primary_value = each_grid_point[0]
        grid_cycle.secondary_value = each_grid_point[1]

        grid_cycle.precision = comparison_measures.precision
        grid_cycle.recall = comparison_measures.recall
        grid_cycle.f1_measure = comparison_measures.f1_measure
        grid_cycle.f05_measure = comparison_measures.f05_measure

        grid_session.add(grid_cycle)
        
        last_grid_entry = grid_session.query(GridTable).order_by(desc(GridTable.id)).first()
        last_grid_entry.completed_grid_id = grid_cycle.id
        
        grid_session.merge(last_grid_entry)
        grid_session.commit()
        
        return grid_cycle
        

    def enter_parameters_in_grid_database(self, primary_variable, secondary_variable, run_name, each_grid_point,
    grid_name, comp_measures):
        grid_session = SpringDataBase().setup_sqlite_db(grid_base, grid_name)
                
        this_cycle = self.make_database_entries(run_name, each_grid_point, grid_session, comp_measures)
        
        columns = SpringDataBase().get_columns_from_table(GridRefineTable)
        log_entries = [str(getattr(this_cycle, each_col)) for each_col in columns]
                                                         
        msg = tabulate(zip(columns, log_entries))

        self.log.ilog('The following parameters have been determined and entered into the database:\n{0}'.format(msg))
        
    
    def enter_variables_into_grid_database(self, primary_variable, secondary_variable, primary_range, secondary_range,
    primary_inc, second_inc, rise_rot_or_pitch_unit_choice=None):
        """
        common?
        """
        if rise_rot_or_pitch_unit_choice is None:
            primary_db_variable, secondary_db_variable = primary_variable, secondary_variable
        else:
            primary_db_variable, secondary_db_variable = \
            SegClassReconstruct().get_correct_primary_and_secondary_variables_for_database(rise_rot_or_pitch_unit_choice, 
            secondary_variable)
            
        this_grid_run = SegClassReconstruct().enter_starting_parameters_of_grid_search(primary_db_variable, 
            secondary_db_variable, primary_range, primary_inc, secondary_range, second_inc)
        
        return this_grid_run, primary_db_variable, secondary_db_variable
    

    def prepare_grid_database(self, primary_variable, secondary_variable, primary_range, secondary_range, primary_inc,
    second_inc, grid_name):
        grid_session = SpringDataBase().setup_sqlite_db(grid_base, grid_name)
        this_grid_run, primary_db_variable, secondary_db_variable = self.enter_variables_into_grid_database(primary_variable, 
            secondary_variable, primary_range, secondary_range, primary_inc, second_inc)
        
        grid_session.add(this_grid_run)
        grid_session.commit()

        return primary_db_variable, secondary_db_variable


    def retrieve_performance_score_from_run(self, trace_db):
        print trace_db
        grid_session = SpringDataBase().setup_sqlite_db(grid_base, trace_db)
        tracing_results = grid_session.query(GridRefineTable).order_by(desc(GridRefineTable.id)).first()
        
        return tracing_results


    def launch_michelixtrace_jobs(self):
        grid_sequence, last_grid_cycle_id, primary_variable, secondary_variable, grid_name = self.setup_grid_sequence()
        
        michelixtrace_parameters = \
        self.generate_michelixtrace_parameters_while_removing_grid_parameters(self.feature_set.parameters)
        
        primary_db_variable, secondary_db_variable = self.prepare_grid_database(primary_variable,
        secondary_variable, self.primary_range, self.secondary_range, self.primary_inc, self.second_inc, grid_name)
            
        for each_grid_id, each_grid_point in enumerate(grid_sequence):
            if each_grid_id >= last_grid_cycle_id:
                parameter_file, run_name = self.setup_parameters_for_each_grid_point(primary_variable, secondary_variable,
                each_grid_point, michelixtrace_parameters)
                
                cmdline = self.generate_cmdline_for_each_grid_point(parameter_file)
                self.log.in_progress_log()
                OpenMpi().launch_command(cmdline)
                
                comp_measures = self.retrieve_performance_score_from_run(os.path.join(run_name, 'trace_grid.db'))
    
                self.enter_parameters_in_grid_database(primary_db_variable, secondary_db_variable, run_name, 
                each_grid_point, grid_name, comp_measures)
            
            self.log.plog(100 * (each_grid_id + 1) / len(grid_sequence))
        
        self.log.endlog(self.feature_set)
            
            
def main():
    # Option handling
    parset = MicHelixTraceGridPar()
    mergeparset = OptHandler(parset)

    ######## Program
    stack = MicHelixTraceGrid(mergeparset)
    stack.launch_michelixtrace_jobs()

if __name__ == '__main__':
    main()


# Author: Carsten Sachse 25-Dec-2013
# Copyright: EMBL (2010 - 2018)
# License: see license.txt for details

from spring.csinfrastr.csproductivity import OpenMpi
from spring.segment2d.segclassexam import SegClassExam, SegClassExamPar
import os


class SegClassExamMpi(SegClassExam):
    
    def prepare_segclassexam_mpi(self):
        self.comm, self.rank, self.size, self.log, self.tempdir = OpenMpi().setup_mpi_and_simultaneous_logging(self.log,
        self.feature_set.logfile, self.temppath)
        
        classno_start, classno_end = self.prepare_segclassexam()
        if self.rank == 0:
            classes = list(range(classno_start, classno_end + 1))
            if self.size > len(classes):
                msg = 'You have requested more CPUs than number of classes to be examined. For fair CPU usage, ' + \
                'reduce number of CPUs to number of classes.'
                raise ValueError(msg)
            classes = OpenMpi().split_sequence_evenly(classes, self.size)
        else:
            classes = None
        
        classes = self.comm.scatter(classes, root=0)

        return classes[0], classes[-1]


    def collect_powers_and_print_figures_and_finish_mpi(self, figures):

        power_img, power_img_enh = self.get_local_power_stacks()
        OpenMpi().gather_stacks_from_cpus_to_common_stack(self.comm, power_img, self.power_img)
        OpenMpi().gather_stacks_from_cpus_to_common_stack(self.comm, power_img_enh, self.power_enhanced_img)
    
        self.print_figures(figures)
        self.comm.barrier()

        os.rmdir(self.tempdir)
        if self.rank == 0:
            self.log.endlog(self.feature_set)


    def get_local_power_stacks(self):
        power_img = os.path.join(self.tempdir, self.power_img)
        power_img_enh = os.path.join(self.tempdir, self.power_enhanced_img)
        return power_img, power_img_enh

    def exam_classes(self):
        classno_start, classno_end = self.prepare_segclassexam_mpi()
        power_img, power_img_enh = self.get_local_power_stacks()

        figures = self.perform_class_examination(self.avgstack, self.varstack, classno_start, classno_end, power_img,
        power_img_enh)

        self.collect_powers_and_print_figures_and_finish_mpi(figures)

def main():
    parset = SegClassExamPar()
    reduced_parset = OpenMpi().start_main_mpi(parset)
    
    ####### Program
    stack = SegClassExamMpi(reduced_parset)
    stack.exam_classes()

if __name__ == '__main__':
    main()



#!/usr/bin/env python
"""
Test module to check segmentrefine3d
"""
from EMAN2 import EMData, EMUtil, Util, Transform
from fundamentals import rot_shift2D, image_decimate
from glob import glob
from morphology import ctf_img
from multiprocessing import cpu_count
from projection import prj
from spring.csinfrastr.csdatabase import SpringDataBase, base, CtfMicrographTable, HelixTable, SegmentTable, \
    CtfFindMicrographTable, CtfTiltMicrographTable, refine_base, RefinementCycleTable
from spring.csinfrastr.csproductivity import TestingDirectory
from spring.segment2d.segment import Segment
from spring.segment2d.segmentalign2d import SegmentAlign2d
from spring.segment3d.refine.sr3d_diagnostics import EulerPlot
from spring.segment3d.refine.sr3d_main import SegmentRefine3d, SegmentRefine3dPar
from spring.segment3d.segclassreconstruct import SegClassReconstruct
from spring.springgui.spring_launch import DataBaseUpdate
from spring.tests.csinfrastr.test_csreadinput import EndToEndTest
from spring.tests.segment3d.test_segclassreconstruct import TestSegClassReconstructPreparation
from sqlalchemy.sql.expression import desc
from statistics import ccc
from utilities import set_params_proj, generate_ctf, get_params_proj, model_circle, model_blank, compose_transform2, \
    model_gauss_noise, model_square
import matplotlib.pyplot as plt
import numpy as np
import os
import random
import shutil
import sys


class TestSegmentRefine3dPreparationDatabase(object):
    def setup_micrograph(self, pixelsize):
        micrograph = CtfMicrographTable()
        micrograph.amplitude_contrast = 0.1
        micrograph.ctffind_determined = True
        micrograph.ctftilt_determined = True
        micrograph.dirname = os.path.abspath(os.curdir)
        micrograph.micrograph_name = 'test_mic.hdf'
        micrograph.pixelsize = pixelsize
        micrograph.spherical_aberration = 2.0
        micrograph.voltage = 200
    
        ctffind_mic = CtfFindMicrographTable()
        ctffind_mic.dirname = os.path.abspath(os.curdir)
        ctffind_mic.micrograph_name = 'test_mic.hdf'
        ctffind_mic.pixelsize = pixelsize
        ctffind_mic.defocus_avg = 15000
        ctffind_mic.defocus1 = 12500
        ctffind_mic.defocus2 = 17500
        ctffind_mic.astigmation_angle = 35
        ctffind_mic.cc_score = 3233
        ctffind_mic.ctf_micrographs = micrograph
        
        ctftilt_mic = CtfTiltMicrographTable()
        ctftilt_mic.defocus_avg = 15500
        ctftilt_mic.defocus1 = 13000
        ctftilt_mic.defocus2 = 18000
        ctftilt_mic.astigmation_angle = 40
        ctftilt_mic.tilt_angle = 0
        ctftilt_mic.tilt_axis= 135.10
        ctftilt_mic.center_x = 500
        ctftilt_mic.center_y = 500
        
        ctftilt_mic.ctffind_ids = ctffind_mic
        ctftilt_mic.ctf_micrographs = micrograph
        
        return micrograph, ctftilt_mic
    
    
    def setup_helix(self, parameters, micrograph, pixelsize, inplane_angles, max_dist):
        phi, theta, psi, shift_x, shift_y = np.hsplit(parameters, 5)
        
        helix = HelixTable()
        helix.avg_curvature = abs(random.gauss(0.001, 0.001))
        helix.avg_inplane_angle = inplane_angles.mean()
        helix.avg_theta = theta.mean()
        helix.dirname = os.path.abspath(os.path.curdir)
        helix.helix_name = 'helix1'
        helix.micrographs = micrograph
        helix.length = max_dist
        helix.ccc_layer_position_start = 0.1
        helix.ccc_layer_position_end = 0.3
        
        return helix
        
    
    def setup_segments(self, seg_id, parameters, micrograph, helix, pixelsize, coord, ip_angle, dist):
        phi, theta, psi, shift_x, shift_y = np.hsplit(parameters, 5)
        
        segment = SegmentTable()
        segment.astigmation_angle = random.randint(0,180)
        segment.astigmatism = abs(random.gauss(0, 1000))
        segment.avg_defocus = abs(random.gauss(20000, 5000))
        segment.ccc_layer = random.gauss(0.5, 0.1)
        segment.ccc_prj = random.gauss(0.5, 0.1)
        segment.class_id = random.randint(0, 3)
        segment.class_model_id = random.randint(0, 3)
        segment.ctffind_applied = False
        segment.ctftilt_applied = True
        segment.ctf_convolved = True
        segment.ctf_phase_flipped = False
        segment.curvature = abs(random.gauss(0.001, 0.001))
        segment.inplane_angle = ip_angle
        x_coord, y_coord = coord
        segment.second_order_fit = None #1.0e-02
        segment.x_coordinate_A = x_coord
        segment.y_coordinate_A = y_coord
        segment.picked_x_coordinate_A = x_coord + (20 * pixelsize)
        segment.picked_y_coordinate_A = y_coord
        segment.distance_from_start_A = dist
        segment.lavg_inplane_angle = ip_angle
        segment.psi = psi
        segment.theta = theta
        segment.phi = phi
        segment.stack_id = seg_id
        segment.helices = helix
        segment.micrographs = micrograph
    
        return segment
    
        
    def compute_inplane_angles_and_distance_from_start(self, x_coordinates, y_coordinates):
        x_y_coord = zip(x_coordinates, y_coordinates)
        
        inplane_angles = (-np.rad2deg(np.arctan(np.diff(x_coordinates) / np.diff(y_coordinates)))) % 360
        inplane_angles = np.append(inplane_angles, inplane_angles[-1])
        
        distances = np.sqrt(np.diff(x_coordinates) ** 2 + np.diff(y_coordinates) ** 2)
        distances = np.insert(distances, 0, 0)
        distance_from_start = np.cumsum(distances)
        
        return x_y_coord, inplane_angles, distance_from_start


    def add_parameters_to_segment(self, parameters, pixelsize, session, micrograph, helix, x_y_coord, inplane_angles,
    distance_from_start, start_id=0):
        for seg_id, each_param_set in enumerate(parameters):
            coord = x_y_coord[seg_id]
            ip_angle = inplane_angles[seg_id]
            dist = distance_from_start[seg_id]

            segment = self.setup_segments(seg_id + start_id, each_param_set, micrograph, helix, pixelsize, coord,
            ip_angle, dist)

            session.add(segment)
        
        return session
    

    def prepare_database_for_refinement(self, parameters, pixelsize, coordinates, start_id=0):
        """
        >>> from spring.tests.segment3d.test_segmentrefine3d import TestSegmentRefine3dPreparationDatabase
        >>> tsar = TestSegmentRefine3dPreparationDatabase()
        >>> tsar.prepare_database_for_refinement(np.array([[33, 90, 0, 1, 2], [34, 90, 135, -1, -2]]), 5, ((np.arange(2), np.arange(2))))
        >>> if os.path.isfile(os.path.join(os.path.pardir, 'spring.db')): os.remove(os.path.join(os.path.pardir, 'spring.db'))
        """
        session = SpringDataBase().setup_sqlite_db(base, os.path.join(os.pardir, 'spring.db'))
        
        micrograph, ctftilt_mic = self.setup_micrograph(pixelsize)
        session.add(ctftilt_mic)
        
        x_coordinates, y_coordinates = coordinates
        
        x_y_coord, inplane_angles, distance_from_start =\
        self.compute_inplane_angles_and_distance_from_start(x_coordinates, y_coordinates)
        
        helix = self.setup_helix(parameters, micrograph, pixelsize, inplane_angles, max(distance_from_start))
        
        session = self.add_parameters_to_segment(parameters, pixelsize, session, micrograph, helix, x_y_coord,
        inplane_angles, distance_from_start, start_id)
        
        session.commit()
            
        
class TestSegmentRefine3dPreparation(TestSegmentRefine3dPreparationDatabase):
    def apply_random_rotations_x_and_y_shifts_to_projection_parameters(self, projection_parameters, x_range):
        perturbed_parameters = np.zeros((len(projection_parameters), 5))
        for each_projection_index, (each_phi, each_theta, each_psi, each_x, each_y) in enumerate(projection_parameters):
            if each_projection_index <= len(projection_parameters) / 2:
                inplane_angle = 45.0
            else:
                inplane_angle = 45.0# + 90.0
                
            each_psi = 270 + inplane_angle 

            each_x, each_y = SegClassReconstruct().compute_sx_sy_from_shifts_normal_and_parallel_to_helix_axis_with_inplane_angle(0.5*x_range, 0, -inplane_angle)
#            each_x = 4.0
#            each_y = -2.0

            perturbed_parameters[each_projection_index]=np.array([each_phi, each_theta, each_psi, each_x, each_y], dtype=float)
        
        return perturbed_parameters
            
            
    def generate_3d_ctf_file(self, ctf_3d, volume_dimension):
        
        avg_defocus=20000.0
        voltage=200
        cs=2.0
        ampcont=0.1
        
        ctf = generate_ctf([avg_defocus, cs, voltage, self.pixelsize, 0, ampcont])
        
        ctf_volume = ctf_img(volume_dimension, ctf, 1, volume_dimension, volume_dimension)
        ctf_volume.write_image(ctf_3d)
        
    
    def clean_projection_parameters_from_input_stack(self, input_stack):
        image_count = EMUtil.get_image_count(input_stack)
        
        img = EMData()
        for each_image_number in range(image_count):
            img.read_image(input_stack, each_image_number)
            determined = get_params_proj(img)
            set_params_proj(img, [determined[0], determined[1], determined[2], -0., -0.])
            img.write_image(input_stack, each_image_number)
            
        return input_stack
    

    def setup_for_all_options_true(self, helix_volume):
        from spring.tests.segment2d.test_segmentclass import HelixClassPreparation
        if self.all_options is True:
            helix_volume.write_image(self.test_reference_volume)
            self.generate_3d_ctf_file(self.ctf_3d_avg_file, self.alignment_size)
            HelixClassPreparation().prepare_custom_filter_file(self.custom_filter_file, self.alignment_size)
            for each_key in self.feature_set.parameters:
                if type(self.feature_set.parameters[each_key]) is bool and each_key != 'MPI option':
                    self.feature_set.parameters[each_key] = True

    def generate_input_stack_from_helix(self):
        segment_size = int(self.volume_size_in_angstrom/self.pixelsize)
        
        self.euler_list = SegmentRefine3d().generate_Euler_angles_for_projection(self.azimuthal_count,
        self.out_of_plane_range, self.out_of_plane_count, self.helical_rotation)
        
        self.perturbed_alignment_parameters = \
        self.apply_random_rotations_x_and_y_shifts_to_projection_parameters(self.euler_list,
        self.x_translation/self.pixelsize)
        
        coord = np.arange(len(self.perturbed_alignment_parameters) / 2) * 10 * self.pixelsize
        coordinates = ((coord, coord))
        half_params = self.perturbed_alignment_parameters[len(self.perturbed_alignment_parameters)/2:]
        self.prepare_database_for_refinement(half_params, self.pixelsize, coordinates)
        
        coord = np.arange(len(self.perturbed_alignment_parameters) / 2, 
                          len(self.perturbed_alignment_parameters)) * 10 * self.pixelsize
        
        coordinates = ((coord, coord))
#         coordinates = ((coord, -1 * coord))
        other_half_params = self.perturbed_alignment_parameters[:len(self.perturbed_alignment_parameters)/2]
        
        self.prepare_database_for_refinement(other_half_params, self.pixelsize, coordinates,
        len(self.perturbed_alignment_parameters)/2)
        
        helix_volume =\
        TestSegClassReconstructPreparation().generate_helical_volume_square(self.size_of_subunit_in_angstrom,
        self.pixelsize, segment_size, self.helix_radius_in_angstrom, self.helical_rise, self.helical_rotation)
        
        if self.prepare_apolar_helix:
            point_symmetry = SegClassReconstruct().determine_point_group_symmetry_from_input('apolar', self.helix_start)
        else:
            point_symmetry = SegClassReconstruct().determine_point_group_symmetry_from_input('polar', self.helix_start)
        self.helix_volume = helix_volume.symvol(point_symmetry)
#            self.helix_volume = SegClassReconstruct().impose_apolar_symmetry_on_helix(self.helix_volume)
#        self.helix_volume.write_image('test_volume.hdf')
        
        TestSegClassReconstructPreparation().generate_noise_projections_from_helix(self.input_stack, self.helix_volume,
        self.perturbed_alignment_parameters)
        
        self.input_stack = self.clean_projection_parameters_from_input_stack(self.input_stack)
        

class TestSegmentRefine3dClean(TestSegmentRefine3dPreparation):
    def generate_iteration_filename_and_remove(self, file_prefix_name, each_iteration):
        file_name_iteration = '{prefix}*{iter:03}{ext}'.format(prefix=os.path.splitext(file_prefix_name)[0],
        iter=each_iteration + 1, ext=os.path.splitext(file_prefix_name)[-1])
        
        files = glob(file_name_iteration)
        for each_file in files:
            if os.path.isfile(each_file):
                os.remove(each_file)
            
        return file_name_iteration


    def clean_files_from_iterations(self):
        os.remove('spring.db')
        for each_iteration in list(range(self.test_iteration_count)):
            self.generate_iteration_filename_and_remove('refinement.db', each_iteration)
            self.generate_iteration_filename_and_remove(self.test_reference_volume, each_iteration)
            self.generate_iteration_filename_and_remove('fsc.dat', each_iteration)
            self.generate_iteration_filename_and_remove('fsc_indpndnt.png', each_iteration)
            self.generate_iteration_filename_and_remove('exp_vs_reproj.png', each_iteration)
            self.generate_iteration_filename_and_remove(self.diagnostic_plot_file, each_iteration)

        if os.path.isfile(self.test_reference_volume):
            os.remove(self.test_reference_volume)

    
    def cleanup_segmentrefine3d(self):
        os.remove(self.input_stack)
        os.remove(os.path.join(os.path.pardir, 'spring.db'))

        self.clean_files_from_iterations()
        
        if self.all_options:
            os.remove(self.custom_filter_file)
            os.remove(self.ctf_3d_avg_file)

    def prepare_final_prj_for_documentation(self, models, outfile=None):
        each_vol = EMData()
        max_dims = []
        for each_model in models:
            each_vol.read_image(each_model)
            max_dims.append(max(each_vol.get_xsize(), each_vol.get_ysize(), each_vol.get_zsize()))
        
        max_dim = max(max_dims)
        for each_model in models: 
            each_vol.read_image(each_model)
            each_vol.process_inplace('normalize')
            each_vol = Util.pad(each_vol, max_dim, max_dim, max_dim, 0, 0, 0, 'average')
            projection_parameters = [[0., 90., 270., 0., 0.]]
            helical_projections = prj(each_vol, projection_parameters, each_model)
            
        if outfile is None:
            outfile = models[0]
        for each_model_id, each_model in enumerate(models):
            each_vol.read_image(each_model)
            each_vol.write_image(outfile, each_model_id)
            if each_model_id > 0:
                os.remove(each_model)


class TestSegmentRefine3dSetup(TestSegmentRefine3dClean):
    def setup_input_output_iteration(self):
        self.feature_set.parameters['Image input stack refinement']=self.input_stack
        self.feature_set.parameters['Output volume name']=self.test_reference_volume
        self.feature_set.parameters['spring.db file']=os.path.join(os.pardir, 'spring.db')
        self.feature_set.parameters['Continue refinement option']=False
        self.feature_set.parameters['refinement.db file']='refinement003.db'
            
        self.feature_set.parameters['Diagnostic plot prefix']=self.diagnostic_plot_file
        self.feature_set.parameters['Number of iterations']=self.test_iteration_count
        self.feature_set.parameters['Reference structure option']=False
        self.feature_set.parameters['Reference volume']=os.path.join(os.pardir, 'test_recvol_100apix_003.hdf')
        self.feature_set.parameters['Keep intermediate files']=False
        
    def setup_helical_symmetry_parameters(self):
        self.feature_set.parameters['Pixel size in Angstrom']=self.pixelsize
        self.feature_set.parameters['Estimated helix inner and outer diameter in Angstrom']=((50, 170))
        self.feature_set.parameters['Symmetrize helix']=True
        self.feature_set.parameters['Enforce even phi option']=False
        self.feature_set.parameters['Release cycle even phi']=6
        self.feature_set.parameters['Pitch enforce even phi']=self.helical_rise * 360.0 / self.helical_rotation
        self.feature_set.parameters['Bin cutoff of phi angles']=10000
        self.feature_set.parameters['Helical rise/rotation or pitch/number of units per turn choice']='rise/rotation'
        
        self.feature_set.parameters['Helical symmetry in Angstrom or degrees']=((self.helical_rise,
        self.helical_rotation))
        
#         self.feature_set.parameters['Refine helical symmetry']=False

    def setup_alignment_parameters(self):
        self.feature_set.parameters['Force helical continuity']=True
        self.feature_set.parameters['Limit in-plane rotation']=True
        self.feature_set.parameters['Delta in-plane rotation angle']=5.0
        self.feature_set.parameters['Out-of-plane tilt angle range']=self.out_of_plane_range
        
        self.feature_set.parameters['Number of projections azimuthal/out-of-plane angle']=((self.azimuthal_count,
        self.out_of_plane_count))


    def setup_selection_criteria_from_segment_table(self, feature_set):
        feature_set.parameters['Segments select option'] = False
        feature_set.parameters['Include or exclude segments'] = 'include'
        feature_set.parameters['Segment file'] = 'test_select_segment.dat'
        
        feature_set.parameters['Micrographs select option'] = False
        feature_set.parameters['Include or exclude micrographs'] = 'include'
        feature_set.parameters['Micrographs list'] = '1'
        
        feature_set.parameters['Helices select option'] = False
        feature_set.parameters['Include or exclude helices'] = 'include'
        feature_set.parameters['Helices list'] = '1'
        
        feature_set.parameters['Classes select option'] = False
        feature_set.parameters['Include or exclude classes'] = 'include'
        feature_set.parameters['Class type'] = 'class_id'
        feature_set.parameters['Classes list'] = '0-2'
        
        feature_set.parameters['Persistence class option'] = False
        feature_set.parameters['Persistence class length in Angstrom'] = 700
        feature_set.parameters['Class occupancy threshold']=0.5

        feature_set.parameters['Straightness select option'] = False
        feature_set.parameters['Include or exclude straight helices'] = 'include'
        feature_set.parameters['Persistence length range'] = (10, 100)
        
        feature_set.parameters['Defocus select option'] = False
        feature_set.parameters['Include or exclude defocus range'] = 'include'
        feature_set.parameters['Defocus range'] = (0, 20000)
        
        feature_set.parameters['Astigmatism select option'] = False
        feature_set.parameters['Include or exclude astigmatic segments'] = 'include'
        feature_set.parameters['Astigmatism range'] = (0, 10000)
        
        feature_set.parameters['Layer line correlation select option'] = False
        feature_set.parameters['Include or exclude segments based on layer-line correlation'] = 'include'
        feature_set.parameters['Correlation layer line range']=((50, 100))
        
        return feature_set
    

    def setup_selection_parameters(self):
        self.feature_set = self.setup_selection_criteria_from_segment_table(self.feature_set)
        
        self.feature_set.parameters['Out-of-plane tilt select option']=False
        self.feature_set.parameters['Include or exclude out-of-plane tilted segments']='include'
        self.feature_set.parameters['Out-of-plane tilt range']=((-8,8))
        
        self.feature_set.parameters['Projection correlation select option']=False
        self.feature_set.parameters['Include or exclude segments based on projection correlation']='include'
        self.feature_set.parameters['Correlation projection range']=((0, 100))
        
        self.feature_set.parameters['Shift normal to helix select option']=False
        self.feature_set.parameters['Include or exclude segments with shift normal to helix']='include'
        self.feature_set.parameters['Shift normal to helix in Angstrom']=10.0
        
    def setup_refinement_strategy_parameters(self):
        self.feature_set.parameters['Absolute X and Y translation limit in Angstrom']= (int(0.5 * self.x_translation),
        int(0.5* self.y_translation))

        self.feature_set.parameters['Assemble refinement strategy']=True
        self.feature_set.parameters['LR - Low resolution aim']=True
        self.feature_set.parameters['LR - azimuthal and out-of-plane search restraint in degrees']=((180.0, 180.0))
        
        self.feature_set.parameters['LR - X and Y translation range in Angstrom']=((self.x_translation,
        self.y_translation))
        
        self.feature_set.parameters['MR - Medium resolution aim']=True
        self.feature_set.parameters['MR - azimuthal and out-of-plane search restraint in degrees']=((80.0, 20.0))
        
        self.feature_set.parameters['MR - X and Y translation range in Angstrom']=((int(self.x_translation / 2.0),
        int(self.y_translation / 2.0)))
        
        self.feature_set.parameters['HR - High resolution aim']=False
        self.feature_set.parameters['HR - azimuthal and out-of-plane search restraint in degrees']=((20.0, 20.0))
        
        self.feature_set.parameters['HR - X and Y translation range in Angstrom']=((int(self.x_translation / 3.0),
        int(self.y_translation / 3.0)))
        
        self.feature_set.parameters['MaxR - Maximum resolution aim']=False
        self.feature_set.parameters['MaxR - azimuthal and out-of-plane search restraint in degrees']= ((2.0, 2.0))
        
        self.feature_set.parameters['MaxR - X and Y translation range in Angstrom']=((int(self.x_translation / 6.0),
        int(self.y_translation / 6.0)))
        
        self.feature_set.parameters['Independent half-set refinement']=False
        self.feature_set.parameters['Half-set refinement start']='medium'
        
        self.feature_set.parameters['Frame motion correction']=False
        self.feature_set.parameters['Frame average window size']= 3
        self.feature_set.parameters['Frame local averaging distance']= 700

        
    def setup_filter_parameters(self):
        self.feature_set.parameters['High-pass filter option']=False
        self.feature_set.parameters['Low-pass filter option']=True
        self.feature_set.parameters['High and low-pass filter cutoffs in 1/Angstrom']=((0.01, 0.1))
        self.feature_set.parameters['Custom filter option']=False
        self.feature_set.parameters['Custom-built filter file']=self.custom_filter_file
        self.feature_set.parameters['Automatic FSC filter']=True
        self.feature_set.parameters['B-Factor']=0
        self.feature_set.parameters['Filter layer-lines option']=False
        

    def setup_reconstruction_parameters(self):
        self.feature_set.parameters['Rotational symmetry']=self.helix_start
        self.feature_set.parameters['Helix polarity']='polar'
        self.feature_set.parameters['Unbending option']=False#True
        self.feature_set.parameters['Image alignment size in Angstrom']=280
        self.feature_set.parameters['Step size of segmentation in Angstrom']=70
        self.feature_set.parameters['Choose out-of-plane tilt amplitude correlation']=False
        self.feature_set.parameters['Amplitude correlation out-of-plane tilt range']=((-10, 10))
        self.feature_set.parameters['3D CTF correction']=False
        self.feature_set.parameters['3D CTF correction intensity']='low'
        

    def setup_main_segmentrefine3d_parameters(self):
        self.test_reference_volume = 'test_recvol.hdf'
        self.diagnostic_plot_file = 'test_diagnostic_plot.png'
        self.ctf_3d_avg_file = 'test_3d_ctfaverage.hdf'
        self.custom_filter_file = 'test_custom_filter_file.dat'
        self.all_options = False
        self.test_iteration_count = 6
        self.pixelsize = 5.0
        self.helical_rise = 10.0
        self.helical_rotation = 50.0
        self.size_of_subunit_in_angstrom = 50
        self.helix_radius_in_angstrom = 50
        self.volume_size_in_angstrom = 400
        self.x_translation = 50
        self.y_translation = 10
        self.azimuthal_count = 12
        self.out_of_plane_range = (-8, 8)
        self.out_of_plane_count = 5
        self.helix_start = 1
        self.setup_input_output_iteration()
        self.setup_helical_symmetry_parameters()
        self.setup_refinement_strategy_parameters()
        self.setup_alignment_parameters()
        self.setup_selection_parameters()
        self.setup_filter_parameters()
        self.setup_reconstruction_parameters()

    def setup_of_common_non_mpi_parameters(self):
        self.feature_set = SegmentRefine3dPar()
        self.input_stack = 'test_stack.hdf'
        self.setup_main_segmentrefine3d_parameters()
        
        
class TestSegmentRefine3dSetupForOthers(TestSegmentRefine3dSetup, SegmentRefine3d):
    def setup_sr3d(self):
        self.setup_of_common_non_mpi_parameters()
        self.feature_set.parameters['MPI option']=False
        self.feature_set.parameters['Number of CPUs']=min(3, cpu_count())
        self.feature_set.parameters['Temporary directory']=os.path.abspath(os.curdir)
        
        self.prepare_apolar_helix = False
        self.generate_input_stack_from_helix()
        
        super(TestSegmentRefine3dSetupForOthers, self).__init__(self.feature_set)
        

class TestSegmentRefine3d(TestSegmentRefine3dSetupForOthers):

    def produce_a_projection_of_final_volume(self):
        last_volume_file = self.generate_file_name_with_apix(self.test_iteration_count, self.test_reference_volume,
        self.pixelsize)
        
        self.prepare_final_prj_for_documentation([last_volume_file], self.test_reference_volume)
#         last_volume = EMData()
#         last_volume.read_image(last_volume_file)
#         
#         projection_parameters = [[0., 90., 270., 0., 0.]]
#         helical_projections = prj(last_volume, projection_parameters, self.test_reference_volume)

    def check_determined_alignment_parameters(self, determined_alignment_parameters, perturbed_alignment_parameters):
        for each_determined_index, each_determined_five in enumerate(determined_alignment_parameters):
            each_perturbed_five = perturbed_alignment_parameters[each_determined_index]
            
            sys.stderr.write('\n{determined} = {perturbed}'.format(determined=each_determined_five,
            perturbed=each_perturbed_five))
            
            
    def check_average_peak_value(self, peak_threshold):
        ref_session = SpringDataBase().setup_sqlite_db(refine_base,
        'refinement{0:03}.db'.format(self.test_iteration_count))
        
        last_cycle = ref_session.query(RefinementCycleTable).order_by(desc(RefinementCycleTable.id)).first()
        
        avg_peak_value = last_cycle.mean_peak
        sys.stderr.write('\naverage peak value {0} > {1}'.format(avg_peak_value, peak_threshold))
        assert last_cycle.iteration_id == self.test_iteration_count
        assert avg_peak_value > peak_threshold
        
        
    def setup(self):
        self.testingdir = TestingDirectory(os.path.splitext(os.path.basename(__file__))[0])
        self.testingdir.create()

        self.setup_sr3d()
         

    def teardown(self):
        pass
        self.cleanup_segmentrefine3d()
         
        self.testingdir.remove()


class TestSegmentRefine3dMain(TestSegmentRefine3d):
    def do_test_case_sr1(self):
        """
        * Standard single CPU segmentrefine3d run
        """
        super(TestSegmentRefine3d, self).__init__(self.feature_set)

        self.perform_iterative_projection_matching_and_3d_reconstruction()
        self.produce_a_projection_of_final_volume()
             
        self.check_average_peak_value(1300)

        
class TestSegmentRefine3dEndToEnd(TestSegmentRefine3d):
    def do_end_to_end_test_sr3d_inputfile(self):
        EndToEndTest().do_end_to_end_inputfile(self.feature_set)
 

    def do_end_to_end_test_sr3d_prompt(self):
        EndToEndTest().do_end_to_end_prompt(self.feature_set)
        
        
class TestSegmentRefine3dMore(TestSegmentRefine3d):
    def remove_additional_files(self):
        projection_files = glob('projection_stack*hdf')
        volume_references = glob('{0}*hdf'.format(os.path.splitext(self.test_reference_volume)[0]))
          
        for each_file in (projection_files + volume_references):
            os.remove(each_file)
         
    def do_test_case_sr0(self):
        """
        * Standard single CPU segmentrefine3d run including update refinement db
        """
        self.perform_iterative_projection_matching_and_3d_reconstruction()
                
        self.check_average_peak_value(1300)
        DataBaseUpdate().update_refinement_db('refinement{0:03}.db'.format(self.iteration_count), 'ref.db')
        os.remove('ref.db')
    
            
    def do_test_case_sr2(self):
        """
        * Segmentrefine3d test run with apolar helix (D2 - symmetry)
        """
        self.prepare_apolar_helix = True
        os.remove(os.path.pardir + os.sep + 'spring.db')
        self.helix_start = 2
        self.generate_input_stack_from_helix()
        self.feature_set.parameters['Helix polarity']='apolar'
        self.feature_set.parameters['HR - High resolution aim']=True
        self.feature_set.parameters['MaxR - Maximum resolution aim']=True
            
        super(TestSegmentRefine3d, self).__init__(self.feature_set)
        self.perform_iterative_projection_matching_and_3d_reconstruction()
        self.check_average_peak_value(1300)
            
            
    def do_test_case_sr3(self):
        """
        * Segmentrefine3d test run with left-handed helical symmetry
        and particular included classes
        """
        self.feature_set.parameters['Helical symmetry in Angstrom or degrees']=((self.helical_rise,
        -self.helical_rotation))
            
        self.feature_set.parameters['Classes select option'] = True
        self.feature_set.parameters['Include or exclude classes'] = 'include'
        self.feature_set.parameters['Classes list'] = '0,1,3,4,5,6,8,9,11,12'
            
        self.feature_set.parameters['3D CTF correction']=True
        self.feature_set.parameters['3D CTF correction intensity']='high'
            
        super(TestSegmentRefine3d, self).__init__(self.feature_set)
        self.perform_iterative_projection_matching_and_3d_reconstruction()
        self.check_average_peak_value(1200)
            
    
    def do_test_case_sr4(self):
        """
        * Segmentrefine3d test run with small image alignment size
        """
        self.feature_set.parameters['Force helical continuity']=False
        self.feature_set.parameters['Image alignment size in Angstrom']=180
        self.feature_set.parameters['Independent half-set refinement']=True
        super(TestSegmentRefine3d, self).__init__(self.feature_set)
        self.perform_iterative_projection_matching_and_3d_reconstruction()
            
        self.check_average_peak_value(650)
           
           
    def do_test_case_sr5(self):
        """
        * Segmentrefine3d test run without assumption of symmetry and attempt to enforce even phi angles
        """
        self.feature_set.parameters['Symmetrize helix']=False
        self.feature_set.parameters['Enforce even phi option']=True
        super(TestSegmentRefine3d, self).__init__(self.feature_set)
        self.perform_iterative_projection_matching_and_3d_reconstruction()
            
        self.check_average_peak_value(1300)
            
           
    def do_test_case_sr6(self):
        """
        * Segmentrefine3d test run with out-of-plane tilt exclusion
        """
        self.feature_set.parameters['Out-of-plane tilt select option']=True
        self.feature_set.parameters['Include or exclude out-of-plane tilted segments']='include'
        self.feature_set.parameters['Out-of-plane tilt range']=((-4,4))
            
        super(TestSegmentRefine3d, self).__init__(self.feature_set)
        self.perform_iterative_projection_matching_and_3d_reconstruction()
            
        self.check_average_peak_value(1300)
          
          
    def do_test_case_sr7_continue(self):
        """
        * Segmentrefine3d test run with continue refinement option
        """
        self.feature_set.parameters['Number of iterations']=3
        self.feature_set.parameters['MR - Medium resolution aim']=False
          
        super(TestSegmentRefine3d, self).__init__(self.feature_set)
        self.perform_iterative_projection_matching_and_3d_reconstruction()
    
        shutil.copy('test_recvol_100apix_003.hdf', os.pardir)
          
        self.feature_set.parameters['Reference structure option']=False
        self.feature_set.parameters['Continue refinement option']=True
        self.feature_set.parameters['LR - Low resolution aim']=False
        self.feature_set.parameters['MR - Medium resolution aim']=True
          
        super(TestSegmentRefine3d, self).__init__(self.feature_set)
        self.perform_iterative_projection_matching_and_3d_reconstruction()
          
        self.check_average_peak_value(1300)
        os.remove(os.path.join(os.pardir, 'test_recvol_100apix_003.hdf'))
          
          
    def do_test_case_sr8(self):
        """
        * Segmentrefine3d test run with custom filter file option
        """
        self.feature_set.parameters['Custom filter option']=True
            
        super(TestSegmentRefine3d, self).__init__(self.feature_set)
        from spring.tests.segment2d.test_segmentclass import HelixClassPreparation
        HelixClassPreparation().prepare_custom_filter_file(self.custom_filter_file, 
        int(round(self.alignment_size_in_A / self.pixelsize)))
            
        self.perform_iterative_projection_matching_and_3d_reconstruction()
            
        self.check_average_peak_value(1100)
            
        os.remove(self.custom_filter_file)
         
 
    def do_test_case_with_amp_corr_tilt_range(self):
        """
        * Standard single CPU segmentrefine3d run with chosen out-of-plane tilt amplitude correlation
        """
        self.feature_set.parameters['Choose out-of-plane tilt amplitude correlation']=True
 
        super(TestSegmentRefine3d, self).__init__(self.feature_set)
        self.perform_iterative_projection_matching_and_3d_reconstruction()
             
        self.check_average_peak_value(1300)


#=======================================================================================================================
#     def do_test_case_with_symmetry_refinement_based_on_amp_corr(self):
#         """
#         * Single CPU segmentrefine3d run including symmetry refinement
#         """
#         self.feature_set.parameters['Refine helical symmetry']=True
#         super(TestSegmentRefine3d, self).__init__(self.feature_set)
# 
#         self.perform_iterative_projection_matching_and_3d_reconstruction()
#         self.produce_a_projection_of_final_volume()
#=======================================================================================================================


class TestSegmentRefine3dFrames(TestSegmentRefine3d):
    def setup_frames_for_averaging(self):
        from spring.tests.segment2d.test_segment import TestSegmentFrames
        ts2d = TestSegmentFrames()
        ts2d.setup()
        ts2d.do_test_case_seg101()

        shutil.copy('refinement_0-4frames.db', os.path.join(os.pardir, 'refinement006.db'))
        shutil.copy('spring_0-4frames.db', os.path.join(os.pardir, os.pardir, 'spring.db'))
        shutil.copy(ts2d.test_frame_stack, os.path.join(os.pardir, self.input_stack))
        ts2d.teardown()
        
        self.feature_set.parameters['Frame motion correction']=True

        self.feature_set.parameters['Reference structure option']=False
        self.feature_set.parameters['Number of iterations']=3
        self.feature_set.parameters['Continue refinement option']=True
        self.feature_set.parameters['refinement.db file']='refinement006.db'
        self.feature_set.parameters['LR - Low resolution aim']=False
        self.feature_set.parameters['MR - Medium resolution aim']=True
        
 
    def do_test_frame_processing_with_frame_averaging(self):
        """
        * Segmentrefine3d test with frame averaging option
        """
        self.setup_frames_for_averaging()
   
        super(TestSegmentRefine3d, self).__init__(self.feature_set)
        self.perform_iterative_projection_matching_and_3d_reconstruction()
        self.check_average_peak_value(1300)
    

    def do_test_frame_processing_with_frame_averaging_MPI(self):
        """
        * Segmentrefine3d MPI test with frame averaging option
        """
        self.setup_frames_for_averaging()
  
        self.feature_set.parameters['MPI option'] = True
          
        EndToEndTest().do_end_to_end_inputfile(self.feature_set)
        self.check_average_peak_value(1300)
  
        
class TestSegmentRefine3dUnbending(TestSegmentRefine3d):
    def generate_long_helix(self):
        helix_length = 800
        radius = 4.0
        larger_dimension = 50
        s = model_square(radius, larger_dimension, larger_dimension, helix_length)
        t1 = Transform({'type':'spider', 'psi':33, 'theta':100, 'phi':77, 'scale':1.0})
        small_radius = radius/2.0
        ss = model_square(small_radius, larger_dimension, larger_dimension, helix_length)
        ss.transform(t1)
        vol = s - ss
        
        long_vol = TestSegClassReconstructPreparation().shift_volume_and_helicise(self.pixelsize, 50, self.helical_rise,
        self.helical_rotation, vol)
        
        SegClassReconstruct().project_locally(long_vol, [[0, 82, 270, 0, 0]], 'long_stack.hdf')
        long_image = EMData()
        long_image.read_image('long_stack.hdf')
        os.remove('long_stack.hdf')
        img_size = long_image.get_xsize()
        
        return helix_length, img_size, long_image


    def generate_bent_long_helix(self, helix_length, img_size, long_image):
        self.polynom_curve = [1 / float(helix_length), -1.0, helix_length / 2.0]
        cols, rows, inplane_angles = Segment().compute_bending_path_row_wise(float(img_size), self.polynom_curve, 0)
        coord_y = np.linspace(0, helix_length, 80)
        coord_x = np.polyval(self.polynom_curve, coord_y) #        import matplotlib.pyplot as plt
    #        plt.plot(cols, rows, 'x', coord_x, coord_y, '+')
    #        plt.xlim(0, helix_length)
    #        plt.ylim(0, helix_length)
    #        plt.savefig('test3.png')
        bent_helix = Segment().straighten_segment(long_image, img_size, cols, rows, inplane_angles, mode='bend')
        bent_helix.process_inplace('normalize')
        noise = model_gauss_noise(1, helix_length, helix_length)
        bent_helix += noise
#        bent_helix.write_image('bent_helix.hdf')
        
        return coord_x, coord_y, bent_helix


    def window_segments_from_bent_helix(self, coord_x, coord_y, bent_helix):
        coord_x += 4.0

#        Segment().write_boxfile(coord_x[10:-10], coord_y[10:-10], 100, 'test.box')
        os.remove(self.input_stack)
        for each_id, (each_x, each_y) in enumerate(zip(coord_x, coord_y)):
            segment = Segment().window_segment(bent_helix, each_x, each_y, 60)
            segment.write_image(self.input_stack, each_id)


    def setup_unbending_filament(self):
        helix_length, img_size, long_image = self.generate_long_helix()
        coord_x, coord_y, bent_helix = self.generate_bent_long_helix(helix_length, img_size, long_image)
        trimmed_coord_x = coord_x[10:-10] 
        trimmed_coord_y = coord_y[10:-10] 
        self.window_segments_from_bent_helix(trimmed_coord_x, trimmed_coord_y, bent_helix)
        
        prj_parameters = np.array([ [each_segid * self.helical_rotation, 90.0, 270.0, 0, 0] \
                                   for each_segid in list(range(len(trimmed_coord_x)))])
        
        os.remove(os.path.join(os.pardir, 'spring.db'))
        
        self.prepare_database_for_refinement(prj_parameters, self.pixelsize, ((trimmed_coord_x * self.pixelsize,
        trimmed_coord_y * self.pixelsize)))
        

    def do_test_case_sr10(self):
        """
        * Single CPU: Unbending of curved helix
        """
        self.feature_set.parameters['Unbending option']=True
        super(TestSegmentRefine3d, self).__init__(self.feature_set)
         
        self.setup_unbending_filament()
         
        self.perform_iterative_projection_matching_and_3d_reconstruction()
        self.check_average_peak_value(1100)
         
         
    def do_test_case_sr11(self):
        """
        * Segmentrefine3d: single CPU: Unbending and layer-line filter
        """
        self.feature_set.parameters['Unbending option']=True
        self.feature_set.parameters['Filter layer-lines option']=True
        super(TestSegmentRefine3d, self).__init__(self.feature_set)
         
        self.setup_unbending_filament()
         
        self.perform_iterative_projection_matching_and_3d_reconstruction()
        self.check_average_peak_value(1100)
        
        
class TestSegmentRefine3dMpi(TestSegmentRefine3d):
    pass
    def do_end_to_end_test_sa_inputfile_with_MPI(self):
        """
        * Segmentrefine3d: MPI standard run
        """
        self.feature_set.parameters['MPI option'] = True
         
        EndToEndTest().do_end_to_end_inputfile(self.feature_set)
        self.check_average_peak_value(1300)
   
  
class TestSegmentRefine3dMpiMore(TestSegmentRefine3d):
    def do_end_to_end_test_sa_inputfile_with_MPI_continue_refhalf(self):
        """
        * MPI run continue refinement from independent half-set refinement
        """
        self.feature_set.parameters['MR - azimuthal and out-of-plane search restraint in degrees']=((180.0, 180.0))
        self.feature_set.parameters['Independent half-set refinement']=True
        self.feature_set.parameters['MPI option'] = True
             
        EndToEndTest().do_end_to_end_inputfile(self.feature_set)
       
        shutil.copy('test_recvol_50apix_006.hdf', os.pardir)
              
        self.feature_set.parameters['Independent half-set refinement']=False
        self.feature_set.parameters['Reference structure option']=True
        self.feature_set.parameters['Reference volume']=os.path.join(os.pardir, 'test_recvol_50apix_006.hdf')
        self.feature_set.parameters['Continue refinement option']=True
        self.feature_set.parameters['refinement.db file']='refinement006.db'
        self.feature_set.parameters['LR - Low resolution aim']=False
        self.feature_set.parameters['MR - Medium resolution aim']=True
              
        EndToEndTest().do_end_to_end_inputfile(self.feature_set)
        self.check_average_peak_value(1300)
        self.test_iteration_count = 12

        os.remove(os.path.join(os.pardir, 'test_recvol_50apix_006.hdf'))


    def do_end_to_end_test_sa_inputfile_with_MPI_continue_ref(self):
        """
        * MPI run continue refinement option
        """
        self.feature_set.parameters['MPI option'] = True
        self.feature_set.parameters['Number of iterations']=3
        self.feature_set.parameters['MR - Medium resolution aim']=False
              
        EndToEndTest().do_end_to_end_inputfile(self.feature_set)
        
        shutil.copy('test_recvol_100apix_003.hdf', os.pardir)
              
        self.feature_set.parameters['Independent half-set refinement']=True
        self.feature_set.parameters['Reference structure option']=False
        self.feature_set.parameters['Continue refinement option']=True
        self.feature_set.parameters['LR - Low resolution aim']=False
        self.feature_set.parameters['MR - Medium resolution aim']=True
              
        EndToEndTest().do_end_to_end_inputfile(self.feature_set)
        self.check_average_peak_value(1300)
              
        os.remove(os.path.join(os.pardir, 'test_recvol_100apix_003.hdf'))

               
    def do_end_to_end_test_sa_inputfile_with_MPI_layer_line_filter(self):
        """
        * MPI Test for layer-line filter
        """
        self.feature_set.parameters['Filter layer-lines option']=True
        
        self.feature_set.parameters['MPI option'] = True
                
        EndToEndTest().do_end_to_end_inputfile(self.feature_set)
        self.check_average_peak_value(1300)
        
          
    def do_end_to_end_test_sa_inputfile_with_MPI_unbending(self):
        """
        * MPI Test for unbending
        """
        self.feature_set.parameters['Unbending option']=True
          
        self.feature_set.parameters['MPI option'] = True
                  
        EndToEndTest().do_end_to_end_inputfile(self.feature_set)
        self.check_average_peak_value(1400)
                    
             
    def do_end_to_end_test_sa_inputfile_with_MPI_unbending_and_layer(self):
        """
        * MPI Test for unbending and layer-line filter at the same time
        """
        self.feature_set.parameters['Unbending option']=True
        self.feature_set.parameters['Filter layer-lines option']=True
          
        self.feature_set.parameters['MPI option'] = True
                  
        EndToEndTest().do_end_to_end_inputfile(self.feature_set)
        self.check_average_peak_value(1100)
        
    
    def do_end_to_end_test_sa_inputfile_with_MPI_classes(self):
        """
        * MPI Test for class exclusion
        """
        self.feature_set.parameters['Classes select option'] = True
        self.feature_set.parameters['Include or exclude classes'] = 'include'
        self.feature_set.parameters['Classes list'] = '0,1,3,4,5,6,8,11,12'
               
        self.feature_set.parameters['Out-of-plane tilt select option']=True
        self.feature_set.parameters['Include or exclude out-of-plane tilted segments']='include'
        self.feature_set.parameters['Out-of-plane tilt range']=((-8,4))
               
        self.feature_set.parameters['MPI option'] = True
               
        EndToEndTest().do_end_to_end_inputfile(self.feature_set)
        self.check_average_peak_value(1300)
             
        
    def do_end_to_end_test_with_MPI_choose_amp_corr_tilt(self):
        """
        * MPI Test for choosing out-of-plane tilt amplitude correlation
        """
        self.feature_set.parameters['Choose out-of-plane tilt amplitude correlation']=True
        self.feature_set.parameters['MPI option'] = True
          
        EndToEndTest().do_end_to_end_inputfile(self.feature_set)
        self.check_average_peak_value(1300)

        
    #===================================================================================================================
    # def do_end_to_end_test_sa_inputfile_with_MPI_and_symmetry_refine(self):
    #     """
    #     * Segmentrefine3d: MPI standard run including symmetry refinement
    #     """
    #     self.feature_set.parameters['Refine helical symmetry']=True
    #     self.feature_set.parameters['MPI option'] = True
    #      
    #     EndToEndTest().do_end_to_end_inputfile(self.feature_set)
    #     self.check_average_peak_value(1300)
    #===================================================================================================================
   

class TestSegmentRefine3dBinAlign(object):
    def setup(self):
        self.translation_step = 0.1
        circle_centered = model_circle(2, 600, 600)
        self.sx = self.sy = 22
        circle_shifted = rot_shift2D(circle_centered, 0, self.sx, self.sy)
        
        self.dec_factor = 6.0
        self.small_circle_shifted = image_decimate(circle_shifted, self.dec_factor)
        self.small_circle_centered = image_decimate(circle_centered, self.dec_factor)
    
    def do_test_aligning_binned_images(self):
        translation_series = np.arange(-10, 10, self.translation_step)
        x_translations, y_translations = np.meshgrid(translation_series, translation_series)
        
        translation_pairs = []
        ccc_values = []
        for each_xtrans_index, each_xtranslation in enumerate(x_translations):
            for each_ytrans_index, each_ytranslation in enumerate(y_translations):
                each_xtranslation = x_translations[each_xtrans_index][each_ytrans_index]
                each_ytranslation = y_translations[each_xtrans_index][each_ytrans_index]
                back_shifted_circle = rot_shift2D(self.small_circle_shifted, 0, each_xtranslation, each_ytranslation)
                translation_pairs.append((each_xtranslation, each_ytranslation))
                ccc_values.append(ccc(back_shifted_circle, self.small_circle_centered))
                
        max_ind = np.argmax(ccc_values)
        dec_shifted_x = self.sx/float(self.dec_factor)
        dec_shifted_y = self.sy/float(self.dec_factor)
        
        sys.stderr.write('{0}\t{1}, {2}\t{3}, {4}'.format(ccc_values[max_ind], translation_pairs[max_ind][0],
        translation_pairs[max_ind][1], dec_shifted_x, dec_shifted_y))
    
        margin = 0.3
        assert -dec_shifted_x - margin < translation_pairs[max_ind][0] < -dec_shifted_x + margin
        assert -dec_shifted_y - margin < translation_pairs[max_ind][1] < -dec_shifted_y + margin
    
    def teardown(self):
        pass

class TestSegmentRefine3dCumShiftAlign(object):
    def setup(self):
        self.img_size = 200
        rect_centered = model_blank(self.img_size - 180, self.img_size - 100, 1, 1)
        self.rect_centered = Util.pad(rect_centered, self.img_size, self.img_size, 1, 0, 0, 0, '0')
        self.sx = 22
        self.sy = 2
        self.rot_angle = 50

        self.rect_shifted_rotated = rot_shift2D(self.rect_centered, self.rot_angle, self.sx, self.sy)

        
        self.x_range = 24
        self.y_range = 3
#        self.x_range = 10
#        self.y_range = 21
        self.step_angle = 1.0
        self.step_x = 1.0
        self.delta_psi = 10
    
        self.polar_interpolation_parameters, ring_weights = SegmentAlign2d().prepare_empty_rings(1, self.img_size/2 - 2, 1)
        
        prj = EMData()
        
        self.reference_rings = [SegmentAlign2d().generate_reference_rings_from_image(self.rect_centered,
        self.polar_interpolation_parameters, ring_weights, self.img_size)]
        
        phi = 0.0
        theta = 90.0
        psi = 270.0
        n1 = np.sin(np.deg2rad(theta)) * np.cos(np.deg2rad(phi))
        n2 = np.sin(np.deg2rad(theta)) * np.sin(np.deg2rad(phi))
        n3 = np.cos(np.deg2rad(theta))
            
        self.reference_rings[0].set_attr_dict({'n1':n1, 'n2':n2, 'n3':n3})
        self.reference_rings[0].set_attr('phi', phi)
        self.reference_rings[0].set_attr('theta', theta)
        self.reference_rings[0].set_attr('psi', psi)
            

    def perform_delta_2d_search(self, previous_shift_x, previous_shift_y, x_range, y_range, rot_angle):
        [ang, sxs, s_ys, mirror, matched_reference_id, peak] = \
        Util.multiref_polar_ali_2d_delta(self.rect_shifted_rotated, self.reference_rings, x_range, y_range, self.step_x,
        'F', self.polar_interpolation_parameters, self.center_x + previous_shift_x, self.center_y + previous_shift_y,
        rot_angle, self.delta_psi)
        
        return sxs, s_ys, ang, mirror, peak
    

    def perform_local_2d_search(self, previous_shift_x, previous_shift_y, x_range, y_range):
        fine_step_angle = np.sin(np.deg2rad(0.05))
        zoom_factor = 5.0
        fine_step_x = self.step_x / zoom_factor
        
        fine_x_range = x_range / zoom_factor
        fine_y_range = y_range / zoom_factor
        
        [ang, sxs, s_ys, mirror, xiref, peak] = Util.multiref_polar_ali_2d_local(self.rect_shifted_rotated,
        self.reference_rings, fine_x_range, fine_y_range, fine_step_x, fine_step_angle, 'F',
        self.polar_interpolation_parameters, self.center_x + previous_shift_x, self.center_y + previous_shift_y)
        
        return sxs, s_ys, ang, mirror, peak
    
    
    def do_test_of_cumulative_alignment(self):
        """
        * Simplest restrained 2D alignment 
        
        For each image the following operations are done:
        #. add dummy values of xform.projection for references and image to be aligned
        #. first coarse search with limited in-plane rotation
        #. convert rotate/shift convention back to shift/rotate convention to update shifts
        #. second fine search with smaller search range but finer search step
        #. convert rotate/shift/convention back to shift/rotate convention to update shifts
        """
        self.center_x = self.center_y = self.rect_shifted_rotated.get_xsize() / 2 
        previous_shift_x = previous_shift_y = 0.0
        transform_projection = Transform({'type':'spider','phi':0.0, 'theta':90.0, 'psi':270.0})
        rot_angle = self.rot_angle 
        for each_iteration_id in list(range(10)):
            self.rect_shifted_rotated.set_attr('xform.projection', transform_projection)
        
            local_x_range = max(1.0, self.x_range / float(each_iteration_id + 1))
            local_y_range = max(1.0, self.y_range / float(each_iteration_id + 1))
            
            sxs, s_ys, ang, mirror, peak = self.perform_delta_2d_search(previous_shift_x, previous_shift_y,
            local_x_range, local_y_range, rot_angle)
            
            angb, sxb, syb, ct = compose_transform2(0.0, sxs, s_ys, 1, -ang, 0.0, 0.0, 1)
            
            local_prev_shift_x = previous_shift_x - sxb
            local_prev_shift_y = previous_shift_y - syb
            
            import sys
            
            sys.stderr.write('\n{0}, {1}, {2}, {3}, {4}, {5}, {6}, {7}, {8}'.format(local_x_range, local_y_range,
            previous_shift_x, previous_shift_y, ang, mirror, peak, sxb, syb))
            
            sxs, s_ys, ang, mirror, peak = self.perform_local_2d_search(local_prev_shift_x, local_prev_shift_y,
            local_x_range, local_y_range)
            
            angb, sxb, syb, ct = compose_transform2(0.0, sxs, s_ys, 1, -ang, 0.0, 0.0, 1)
            previous_shift_x = local_prev_shift_x - sxb
            previous_shift_y = local_prev_shift_y - syb
            rot_angle = ang
            
#            import sys
#            sys.stderr.write('\n{0}, {1}, {2}, {3}, {4}, {5}, {6}'.format(ang, previous_shift_x, previous_shift_y, mirror, peak, sxb, syb))
        assert -0.1 < syb < 0.1
#        back_rot = Segment().shift_and_rotate_image(self.rect_shifted_rotated, -angb, -previous_shift_x, -previous_shift_y)
        a, b, c = Segment().convert_shift_rotate_to_rotate_shift_order(ang, -previous_shift_x, -previous_shift_y)
        back_rot = rot_shift2D(self.rect_shifted_rotated, a, b, c)
        added = back_rot + 2 * self.rect_centered
        added.write_image('added.hdf')
        
    def teardown(self):
        os.remove('added.hdf')
        
    
class TestEulerPlot():
    def setup(self):

        if random.randint(0, 1):
            self.phi_angles = np.arange(0, 360, 4)
            self.theta_angles = np.linspace(80, 100, 90)
            self.psi_angles = 270 * np.ones(90)
        else:
            from utilities import even_angles
            angles = even_angles()
            self.phi_angles, self.theta_angles, self.psi_angles = zip(*angles)

        self.fig = plt.figure()
    
        self.euler = EulerPlot()


    def do_euler_angle_test_spher_prj(self):
        """
        * Project Euler angles on 2D sphere histogram
        """

        self.fig = self.euler.plot_euler_angles_on_spherical_projection(self.fig, self.phi_angles, self.theta_angles,
        self.psi_angles)
        

    def do_euler_angle_test_scatter_sphere(self):
        """
        * Project Euler angles on 2D sphere scatter
        """
        self.fig = self.euler.plot_euler_angles_on_spherical_projection_scatter(self.fig, self.phi_angles, self.theta_angles,
        self.psi_angles)


    def do_euler_angle_test_polar_plot_scatter(self):
        """
        * Plot Euler angles on polar plot scatter
        """
        self.fig = self.euler.plot_euler_angles_on_polar_plot_scatter(self.fig, self.phi_angles, self.theta_angles,
        self.psi_angles)
    

    def do_euler_angle_test_polar_plot_hist(self):
        """
        * Plot Euler angles on polar plot histogram
        """
        self.fig = self.euler.plot_euler_angles_on_polar_plot_hist(self.fig, self.phi_angles, self.theta_angles,
        self.psi_angles)

    def do_euler_angle_test_plot_on_3dsphere(self):
        """
        * Plot Euler angles on 3D sphere scatter
        """
        self.fig = self.euler.plot_euler_angles_on_sphere(self.fig, self.phi_angles, self.theta_angles, self.psi_angles)
        

    def do_euler_angle_test_plot_phi_theta_scatter(self):
        """
        * Plot Euler angles in a simple xy hexbin scatter
        """
        self.fig = self.euler.plot_euler_angle_scatter(self.fig, self.phi_angles, self.theta_angles)


    def teardown(self):
        self.fig.savefig('test_euler.png')
        os.remove('test_euler.png')
        
        
def main():
    tsr3d = TestSegmentRefine3dMain()
    tsr3d.setup()
    tsr3d.do_test_case_sr1()
        
if __name__ == '__main__':
    main()
